/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file gpio_device.h
 */

#ifndef __GPIO_DEVICE_H__
#define __GPIO_DEVICE_H__

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include "hdf.h"
#include "errno.h"

#define GPIO_MODE_INPUT		1 << 0  /*!< Bit mask for GPIO input enable */
#define GPIO_MODE_OUTPUT	1 << 1  /*!< Bit mask for GPIO output enable */
#define GPIO_MODE_PULLUP  	1 << 2  /*!< Bit mask for GPIO pull up enable */
#define GPIO_MODE_PULLDOWN    	1 << 3  /*!< Bit mask for GPIO pull down enable */
#define GPIO_MODE_INTERRUPT   	1 << 4  /*!< Bit mask for GPIO interrupt enable */

/**
 * @brief A structure to represent GPIO device.
 */
typedef const struct GPIO_Device_t {
    void (*EnablePullUp)(uint8_t pos, bool bit);
    void (*EnablePullDown)(uint8_t pos, bool bit);
    void (*EnableOutput)(uint8_t pos, bool bit);
    void (*EnableInput)(uint8_t pos, bool bit);
    void (*EnableInterrupt)(uint8_t pos, bool bit);
    void (*Set)(uint8_t pos, uint8_t bit);
    uint8_t (*Read)(uint8_t pos);
    void (*EnablePortPullUp)(bool bit, uint8_t mask);
    void (*EnablePortPullDown)(bool bit, uint8_t mask);
    void (*EnablePortOutput)(bool bit, uint8_t mask);
    void (*EnablePortInput)(bool bit, uint8_t mask);
    void (*EnablePortInterrupt)(bool bit, uint8_t mask);
    void (*PortSet)(uint8_t val, uint8_t mask);
    uint8_t (*PortRead)(uint8_t mask);
    IRQn_Type IRQ;
    char *Name;
} GPIO_Device_t;

/**
 * @brief Request a GPIO Device
 *
 * @param name The name string of GPIO device.
 * @return The pointer to the GPIO device instance.
 */
GPIO_Device_t *GPIO_devRequest(const char *name);

/**
 * @brief Register a callback function to GPIO interrupt handler.
 *
 * @param dev Pointer to the GPIO device.
 * @param cb The callback function to be registered.
 * @return 0 for success or error code upon a failure.
 */
int32_t GPIO_devRegisterIRQ(GPIO_Device_t *dev, void *cb);

/**
 * @brief Unregister a callback function to GPIO interrupt handler.
 *
 * @param dev Pointer to the GPIO device.
 * @param cb The callback function to be unregistered.
 * @return 0 for success or error code upon a failure.
 */
int32_t GPIO_devUnregisterIRQ(GPIO_Device_t *dev, void *cb);

/**
 * @brief Initialize a GPIO pin.
 *
 * @param dev Pointer to the GPIO device
 * @param pos Pin position to the GPIO device.
 * @param mode Bit mask indicating the combination of the mode for this pin.
 * @return 0 for success or error code upon a failure.
 */
int32_t GPIO_devInit(GPIO_Device_t *dev, uint8_t pos, uint32_t mode);

/**
 * @brief Set GPIO pin state.
 *
 * @param dev Pointer to the GPIO device.
 * @param pos Pin position to the GPIO device.
 * @param state Pin state, 0 for low level, none zone for high level.
 * @return 0 for success or error code upon a failure.
 */
int32_t GPIO_devSet(GPIO_Device_t *dev, uint8_t pos, uint8_t state);

/**
 * @brief Read GPIO pin state.
 *
 * @param dev Pointer to the GPIO device.
 * @param pos Pin position to the GPIO device.
 * @return The state of the GPIO pin or error code upon a failure.
 */
int32_t GPIO_devRead(GPIO_Device_t *dev, uint8_t pos);

/**
 * @brief Initialize a GPIO port.
 *
 * @param dev Pointer to the GPIO device
 * @param mask The bit mask in the GPIO port to be configured.
 * @param mode The bit mask indicating the combination of the mode for this port.
 * @return 0 for success or error code upon a failure.
 */
int32_t GPIO_devInitPort(GPIO_Device_t *dev, uint8_t mask, uint32_t mode);

/**
 * @brief Set GPIO port.
 *
 * @param dev Pointer to the GPIO device.
 * @param mask The bit mask in the GPIO port to be set.
 * @param val The value to be programmed to the GPIO port.
 * @return 0 for success or error code upon a failure.
 */
int32_t GPIO_devPortSet(GPIO_Device_t *dev, uint8_t mask, uint8_t val);

/**
 * @brief Read GPIO pin state.
 *
 * @param dev Pointer to the GPIO device.
 * @param mask The bit mask in the GPIO port to be read.
 * @return The masked value of the GPIO port or error code upon a failure.
 */
int32_t GPIO_devPortRead(GPIO_Device_t *dev, uint8_t mask);

#endif /* __GPIO_DEVICE_H__ */