/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file i2c_device.h
 */

#ifndef __I2C_DEVICE_H__
#define __I2C_DEVICE_H__

#include <stdint.h>
#include <stdbool.h>
#include "hdf.h"
#include "errno.h"

#define I2C_DEVICE_BUFFER_SIZE  64

/**
 * @brief The enumeration of I2C Speed
 */
enum {
    I2C_DEVICE_SPEED_100K,
    I2C_DEVICE_SPEED_200K,
    I2C_DEVICE_SPEED_300K,
    I2C_DEVICE_SPEED_400K,
    I2C_DEVICE_SPEED_500K,
    I2C_DEVICE_SPEED_600K,
    I2C_DEVICE_SPEED_MAX = I2C_DEVICE_SPEED_600K,
};

/**
 * @brief A structure to represent I2C slave device initialization information.
 */
typedef struct I2C_Device_SlaveInfo_Type {
    uint8_t             SlaveAddrLen;
    bool                GeneralCallEnable;
    bool                FilterEnable;
    uint16_t            SlaveAddr;    
    uint16_t            SlaveAddrMask;
    uint16_t            Speed;
} I2C_Device_SlaveInfo_t;

/**
 * @brief A structure to represent I2C master device initialization information.
 */
typedef struct I2C_Device_MasterInfo_Type {
    uint8_t             SlaveAddrLen;
    bool                FilterEnable;
    uint16_t            Speed;
} I2C_Device_MasterInfo_t;

/**
 * @brief A structure to represent I2C device state.
 */
typedef struct {
    uint8_t status;
    uint8_t data;
    uint8_t counter;
    uint8_t collision;
    uint8_t address;
    uint8_t state;
    uint8_t read_index;
    uint8_t write_index;
    uint8_t read_len;
    uint8_t write_len;
    uint8_t read_buf[I2C_DEVICE_BUFFER_SIZE];
    uint8_t write_buf[I2C_DEVICE_BUFFER_SIZE];
    uint8_t mode;
    uint8_t opmode;
} I2C_State_t;

/**
 * @brief A structure to represent I2C device.
 */
typedef struct I2C_Device_Type {
    void (*Enable)(void);
    void (*Disable)(void);
    uint8_t (*ReadData)(void);
    uint8_t (*GetStatus)(void);
    void (*WriteData)(uint8_t data);
    void (*SlaveClockStretch)(void);
    void (*SlaveReleaseClock)(void);
    void (*MasterRepeatStart)(void);
    void (*MasterReceiveStart)(void);
    void (*MasterIssueNack)(void);
    void (*MasterIssueAck)(void);
    void (*MasterIssueStop)(void);
    void (*MasterIssueStart)(void);
    uint8_t (*StartBitReceived)(void);
    void (*Set7BitSlaveAddr)(void);
    void (*Set10BitSlaveAddr)(void);
    void (*SetMasterMode)(void);
    void (*SetSlaveMode)(void);
    void (*SlaveEnableGeneralCall)(void);
    void (*SlaveDisableGeneralCall)(void);
    void (*EnableFilter)(void);
    void (*DisableFilter)(void);
    void (*SetSlaveAddress)(uint16_t addr);
    void (*SetSlaveAddressMask)(uint16_t mask);
    void (*SetClockDivider)(uint8_t div);
    IRQn_Type IRQ;
    IRQn_Type CollisionIRQ;
    char *Name;
    I2C_State_t *state;
} I2C_Device_t;

/**
 * @brief Request a I2C Device
 *
 * @param name The name string of I2C device.
 * @return The pointer to the I2C device instance.
 */
I2C_Device_t *I2C_devGetDevice(const char *name);

/**
 * @brief Initialize I2C master device.
 *
 * @param dev The pointer to I2C device
 * @param info The pointer to the initialization information for I2C master device.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devMasterInit(I2C_Device_t *dev, I2C_Device_MasterInfo_t *info);

/**
 * @brief Initialize I2C slave device.
 *
 * @param dev The pointer to I2C device
 * @param info The pointer to the initialization information for I2C slave device.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devSlaveInit(I2C_Device_t *dev, I2C_Device_SlaveInfo_t *info);

/**
 * @brief Reset I2C master device.
 *
 * @param dev The pointer to I2C device
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devReset(I2C_Device_t *dev);

/**
 * @brief Send a data block through I2C master device.
 *
 * @param dev The pointer to I2C device.
 * @param addr The slave addr.
 * @param data The pointer to the data block.
 * @param len The length of the data block.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devMasterSendData(I2C_Device_t *dev, uint16_t addr, uint8_t *data, uint32_t len);

/**
 * @brief Send a read request to I2C master device.
 *
 * @param dev The pointer to I2C device.
 * @param addr The slave addr.
 * @param len The length of the data block.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devMasterReadRequest(I2C_Device_t *dev, uint16_t addr, uint32_t len);

/**
 * @brief Send a register read request to I2C master device.
 *
 * @param dev The pointer to I2C device.
 * @param slave_addr The slave device address.
 * @param reg_addr The pointer to register address in slave device.
 * @param addrlen The length of register address.
 * @param datalen The data length to be read from the slave.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devMasterRegisterReadRequest(I2C_Device_t *dev, uint16_t slave_addr,
    uint8_t *reg_addr, uint8_t addrlen, uint8_t datalen);

/**
 * @brief Read data from I2C master device, the data is the result of last read request.
 *
 * @param dev The pointer to I2C device.
 * @param data The pointer to the memory for saving the data block.
 * @param len The length of the data block.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devMasterReadResult(I2C_Device_t *dev, uint8_t *data, uint32_t len);

/**
 * @brief Check if I2C master device is busy in last transation.
 *
 * @param dev The pointer to I2C device.
 * @return 0 for idle, 1 for busy or error code upon a failure.
 */
int32_t I2C_devIsInProgress(I2C_Device_t *dev);

/**
 * @brief Enable I2C hardware module.
 *
 * @param dev The pointer to I2C device.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devEnable(I2C_Device_t *dev);

/**
 * @brief Disable I2C hardware module.
 *
 * @param dev The pointer to I2C device.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devDisable(I2C_Device_t *dev);

/**
 * @brief Set response data to I2C slave module.
 *
 * @param dev The pointer to I2C device.
 * @param array The pointer to data array to be set.
 * @param len The length of data array.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devSlaveSetResponseData(I2C_Device_t *dev, uint8_t *array, uint32_t len);

/**
 * @brief Read data from I2C slave module.
 *
 * @param dev The pointer to I2C device.
 * @param array The pointer to data array to store the data.
 * @param len The length of data array.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devSlaveReadResult(I2C_Device_t *dev, uint8_t *array, uint32_t len);

/**
 * @brief Check how many bytes received by I2C slave module.
 *
 * @param dev The pointer to I2C device.
 * @return The byte size of received data.
 */
int32_t I2C_devSlaveDataReceived(I2C_Device_t *dev);

/**
 * @brief Check how many bytes has been read by a I2C master.
 *
 * @param dev The pointer to I2C device.
 * @return The byte size of data read by master.
 */
int32_t I2C_devSlaveDataResponsed(I2C_Device_t *dev);

/**
 * @brief Restart next I2C cycle as a slave.
 *
 * @param dev The pointer to I2C device.
 * @return 0 for success or error code upon a failure.
 */
int32_t I2C_devSlaveRestart(I2C_Device_t *dev);

#endif /* __I2C_DEVICE_H__ */