/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file pwm_device.h
  */

#ifndef __PWM_DEVICE_H__
#define __PWM_DEVICE_H__

#include <stdint.h>
#include "hdf.h"

/**
 * @brief A structure to represent a PWM device.
 */
typedef const struct PWM_Device_t {
    void (*SetPeriod)(uint32_t percent);
    void (*SetWidth)(uint32_t percent);
    void (*SetPreScaler)(uint8_t prescaler);
    void (*InvertOn)(void);
    void (*Enable)(void);
    void (*InvertOff)(void);
    void (*Disable)(void);
    char *Name;
} PWM_Device_t;

/**
 * @brief Set period of a PWM driver
 *
 * @param dev The pointer to PWM_Device structure.
 * @param period: percentage of PWM period
 * @return 0 for success or error code upon a failure.
 */
int32_t PWM_devSetPeriod(PWM_Device_t *dev, uint32_t period);

/**
 * @brief Set width of a PWM driver
 *
 * @param dev The pointer to PWM_Device structure.
 * @param width: percentage of PWM width
 * @return 0 for success or error code upon a failure.
 */
int32_t PWM_devSetWidth(PWM_Device_t *dev, uint32_t width);

/**
 * @brief Set pre-scaler of a PWM driver
 *
 * @param dev The pointer to PWM_Device structure.
 * @param div: the setting for clock divider.
 * @return 0 for success or error code upon a failure.
 */
int32_t PWM_devSetClockDivider(PWM_Device_t *dev,  uint8_t div);

/**
 * @brief Enable PWM inverter
 *
 * @param dev The pointer to PWM_Device structure.
 * @return 0 for success or error code upon a failure.
 */
int32_t PWM_devInvertEnable(PWM_Device_t *dev);

/**
 * @brief Disable PWM inverter
 *
 * @param dev The pointer to PWM_Device structure.
 * @return 0 for success or error code upon a failure.
 */
int32_t PWM_devInvertDisable(PWM_Device_t *dev);

/**
 * @brief Enable PWM driver
 *
 * @param dev The pointer to PWM_Device structure.
 * @return 0 for success or error code upon a failure.
 */
int32_t PWM_devEnable(PWM_Device_t *dev);

/**
 * @brief Disable PWM driver
 *
 * @param dev The pointer to PWM_Device structure.
 * @return 0 for success or error code upon a failure.
 */
int32_t PWM_devDisable(PWM_Device_t *dev);

/**
 * @brief Request a PWM Device
 *
 * @param name The name string of PWM device.
 * @return The pointer to the PWM device instance.
 */
PWM_Device_t *PWM_devRequest(const char *name);

#endif /* __PWM_DEVICE_H__  */