/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file uart_device.h
 */

#ifndef __UART_DEVICE_H__
#define __UART_DEVICE_H__

#include <stdint.h>
#include "hdf.h"

#define UART_BUFF_SIZE 256

typedef const struct UART_Device_t UART_Device_Type;
typedef void (*uart_cb_func_t)(UART_Device_Type *dev);

/**
 * @brief Enumeration for UART baud rate
 */
typedef enum {
    UART_DEV_BAUDRATE_300,
    UART_DEV_BAUDRATE_600,
    UART_DEV_BAUDRATE_1200,
    UART_DEV_BAUDRATE_2400,
    UART_DEV_BAUDRATE_4800,
    UART_DEV_BAUDRATE_9600,
    UART_DEV_BAUDRATE_19200,
    UART_DEV_BAUDRATE_38400,
    UART_DEV_BAUDRATE_57600,
    UART_DEV_BAUDRATE_115200,
    UART_DEV_BAUDRATE_230400,
    UART_DEV_BAUDRATE_MAX = UART_DEV_BAUDRATE_230400,
} UART_devBaudRate_t;    

/**
 * @brief Enumeration for UART parity bit
 */
typedef enum {
    UART_DEV_PARITY_BIT_EVEN,
    UART_DEV_PARITY_BIT_ODD,
} UART_Dev_Parity_t;    

/**
 * @brief A structure to represent UART device state.
 */
typedef struct {
    uint32_t buf_size;
    uint32_t rx_outptr;
    uint32_t rx_inptr;
    uint32_t tx_outptr;
    uint32_t tx_inptr;
    uint8_t tx_init;
    uint8_t tx_buffer[UART_BUFF_SIZE];
    uint8_t rx_buffer[UART_BUFF_SIZE];
    uint32_t cb_size;
    uart_cb_func_t cb;
} UART_Dev_State_t;

/**
 * @brief A structure to represent UART initialization information.
 */
typedef struct {
    uint8_t DataSize;
    uint8_t StopBits;
    uint8_t ParityBit;
    uint8_t ParityEn;   
    uint8_t LoopBackEn;
    uint8_t BreakEn;
    uint8_t StickyParityEn;   
} UART_Dev_LineInitData_t;

/**
 * @brief A structure to represent UART device driver.
 */
typedef const struct UART_Device_t {
    void (*LineInit)(UART_Dev_LineInitData_t *data);
    void (*Enable)(void);
    void (*Disable)(void);
    void (*RxFiFoReset)(void);
    void (*TxFiFoReset)(void);
    uint8_t (*LineStatus)(void);
    void (*TxData)(uint8_t data);
    uint8_t (*RxData)(void);
    uint8_t (*IntStatus)(void );
    void (*BaudRateDiv)(uint32_t div);
    void (*EnableRxIRQ)(void);
    void (*EnableTxIRQ)(void);
    void (*DisableRxIRQ)(void);
    void (*DisableTxIRQ)(void);
    void (*EnableRxTimeoutIRQ)(void);
    void (*DisableRxTimeoutIRQ)(void);
    void (*EnableRxErrorIRQ)(void);
    void (*DisableRxErrorIRQ)(void);
    uint8_t (*TxEmpty)(void);
    uint8_t (*RxDataReady)(void);
    void (*SetDataSize)(uint8_t size);
    void (*SetStopBit)(uint8_t stop);
    void (*SetEvenParityBit)(void);
    void (*SetOddParityBit)(void);
    void (*EnableParity)(void);
    void (*DisableParity)(void);
    void (*EnableStickyParity)(void);
    void (*DisableStickyParity)(void);
    void (*EnableBreak)(void);
    void (*DisableBreak)(void);
    void (*EnableLoopBack)(void);
    void (*DisableLoopBack)(void);
    void (*SetInterruptDelay)(uint8_t delay);

    char *Name;
    IRQn_Type IRQ;
    UART_Dev_State_t *state;
    void *SFRS;
    uint16_t *BaudRateMap;
} UART_Device_t;

/**
 * @brief Initialize UART device
 *
 * @param dev Pointer to UART Device structure
 * @param rate Baud Rate
 * @param init Pointer to initial data structure
 * @return 0 for success or error code upon a failure
 */
int32_t UART_devInit(UART_Device_t *dev, UART_devBaudRate_t rate, UART_Dev_LineInitData_t *init);

/**
 * @brief Request an UART device.
 *
 * @param name Name string of the UART device.
 * @return The pointer to the UART device or NULL upon a failure.
 */
UART_Device_t *UART_devRequest(const char *name);

/**
 * @brief Get the number of bytes received.
 *
 * @param dev Pointer to UART Device structure.
 * @return the number of bytes received or error code upon a failure
 */
int32_t UART_devDataReceived(UART_Device_t *dev);

/**
 * @brief Check the number of bytes pending on TX.
 *
 * @param dev Pointer to UART Device structure.
 * @return the number of bytes pending on TX or error code upon a failure
 */
int32_t UART_devDataPendingTX(UART_Device_t *dev);

/**
 * @brief Send data to UART device.
 *
 * @param dev Pointer to UART Device structure.
 * @param data Pointer to the data memory to be sent.
 * @param len The size of the data memory.
 * @return 0 for success or error code upon a failure
 */
int32_t UART_devSendByte(UART_Device_t *dev, uint8_t *data, uint32_t len);

/**
 * @brief Receive data to UART device.
 *
 * @param dev Pointer to UART Device structure.
 * @param data Pointer to the data memory to store the received data.
 * @param len The size of the data memory.
 * @return 0 for success or error code upon a failure
 */
int32_t UART_devReceiveByte(UART_Device_t *dev, uint8_t *data, uint32_t len);

/**
 * @brief Check UART device for error
 *
 * @param dev Pointer to UART Device structure
 * @return 0 for no error or 1 if error occurred.
 */
int32_t UART_devCheckError(UART_Device_t *dev);

/**
 * @brief Reset UART device
 *
 * @param dev Pointer to UART Device structure
 * @return 0 for success or error code upon a failure
 */
int32_t UART_devResetDevice(UART_Device_t *dev);

/**
 * @brief Clear data buffer in UART device driver
 *
 * @param dev Pointer to UART Device structure
 * @return 0 for success or error code upon a failure
 */
int32_t UART_devClearBuffer(UART_Device_t *dev);

/**
 * @brief Register a call back function to UART device driver
 *
 * @param dev Pointer to UART Device structure
 * @param size The size of the data block received to trigger the callback.
 * @param cb The pointer to the call back function.
 * @return 0 for success or error code upon a failure
 */
int32_t UART_devRegisterCallback(UART_Device_t *dev, uint32_t size, void *cb);


#endif /* __UART_DEVICE_H__ */
