/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file adc_device.c
 */

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>
#include <string.h>
#include "adc_device.h"
#include "errno.h"

/**
 * @brief The instance of ADC device
 */
#ifdef ADC_SFRS
static const ADC_Device_t ADC_Device = {
    .SelectChannel = f_ADC_SelectChannel,
    .ClkDiv = f_ADC_ClkDiv,
    .SampleCycle = f_ADC_SampleCycle,
    .Read = f_ADC_Read,
    .Start = f_ADC_Start,
    .SetReferenceHigh = f_ADC_ReferenceHigh,
    .SetReferenceLow = f_ADC_ReferenceLow,
    .SetReferenceGain = f_ADC_ReferenceGain,
    .ConversionInProgress = f_ADC_ConversionInProgress,
    .SetRefSourceVDD = f_ADC_SetRefSourceVDD,
    .SetRefSourceBandGap = f_ADC_SetRefSourceBandGap,
    .SetStreamingMode = f_ADC_EnableStreamingMode,
    .SetSingleMode = f_ADC_DisableStreamingMode,
    .SetCalibrationMode = f_ADC_EnableCalibrationMode,
    .SetNormalMode = f_ADC_DisableCalibrationMode,
    .Name = "ADC",
};
#endif /* ADC_SFRS */

/**
 * @brief The table of ADC device instances supported by the platform.
 */
const static ADC_Device_t *ADC_DeviceTable[] = {
#ifdef ADC_SFRS
    &ADC_Device,
#endif
    NULL,
};

int32_t ADC_devSelectChannel(ADC_Device_t *dev, uint8_t channel)
{
    if (!dev)
        return -EFAULT;
    dev->SelectChannel(channel);
    return 0;
}

int32_t ADC_devRead(ADC_Device_t *dev, uint8_t *val)
{
    if (!dev)
        return -EFAULT;
    *val = dev->Read();
    return 0;
}

int32_t ADC_devStart(ADC_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    dev->Start();
    return 0;
    
}

int32_t ADC_devConversionInProgress(ADC_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return (int32_t)dev->ConversionInProgress();
}

int32_t ADC_devTake(ADC_Device_t *dev, uint8_t channel, uint8_t *val)
{
    if (!dev)
        return -EFAULT;
    
    dev->SelectChannel(channel);
    dev->Start();
    while (dev->ConversionInProgress());
    *val = dev->Read();
    return 0;
}

int32_t ADC_devInit(ADC_Device_t *dev, uint8_t div, uint8_t cycle, uint32_t mode)
{
    if (!dev)
        return -EFAULT;

    if (mode & ADC_MODE_SINGLE && mode & ADC_MODE_STREAM)
        return -EPERM;
    if (mode & ADC_MODE_NORMAL && mode & ADC_MODE_CALIBRATION)
        return -EPERM;
    if (mode & ADC_MODE_REF_VDD && mode & ADC_MODE_REF_BANDGAP)
        return -EPERM;

    dev->ClkDiv(div);
    dev->SampleCycle(cycle);
    if (mode & ADC_MODE_STREAM)
        dev->SetStreamingMode();
    else
        dev->SetSingleMode();
    if (mode & ADC_MODE_CALIBRATION)
        dev->SetCalibrationMode();
    else
        dev->SetNormalMode();
    if (mode & ADC_MODE_REF_BANDGAP)
        dev->SetRefSourceBandGap();
    else
        dev->SetRefSourceVDD();
    return 0;
}

int32_t ADC_devConfigReference(ADC_Device_t *dev, uint8_t refh, uint8_t refl, uint8_t refg)
{
    if (!dev)
        return -EFAULT;
    dev->SetReferenceHigh(refh);
    dev->SetReferenceLow(refl);
    dev->SetReferenceGain(refg);
    return 0;
}

ADC_Device_t *ADC_devRequest(const char *name)
{
    int8_t i;

    for (i = 0; i < sizeof(ADC_DeviceTable)/sizeof(ADC_DeviceTable[0]); i++) {
        if ((ADC_DeviceTable[i] != NULL) &&
            (strncmp(name, ADC_DeviceTable[i]->Name,
                    strlen(ADC_DeviceTable[i]->Name)) == 0))
            return ADC_DeviceTable[i];
    }
    return NULL;
}
