/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file i2c_device.c
 */

#include <string.h>
#include "i2c_device.h"


/**
 * At 12MHz system clock, the clock divider settings for each I2C speed.
 */
static const uint16_t i2c_speed_map[I2C_DEVICE_SPEED_MAX+1] = {
    [I2C_DEVICE_SPEED_100K]	= 59,
    [I2C_DEVICE_SPEED_200K]	= 29,
    [I2C_DEVICE_SPEED_300K]	= 19,
    [I2C_DEVICE_SPEED_400K]	= 14,
    [I2C_DEVICE_SPEED_500K]	= 11,
    [I2C_DEVICE_SPEED_600K]	= 9,
};

/**
 * Enumeration of the states of I2C state machine.
 */
enum {
    I2C_STATE_IDLE,
    I2C_STATE_READ_START,
    I2C_STATE_WRITE_START,
    I2C_STATE_REPEAT_START,
    I2C_STATE_WRITE_DATA,
    I2C_STATE_RECEIVE_DATA,
    I2C_STATE_READ_DATA,
    I2C_STATE_STOP,
};

/**
 * Enumeration of the mode of device.
 */
enum {
    I2C_MODE_MASTER,
    I2C_MODE_SLAVE,
};

/**
 * Enumeration of the mode of operation.
 */
enum {
    I2C_OP_READ,
    I2C_OP_WRITE,
    I2C_OP_REGISTER_READ,
};

/**
 * Enumeration of the read/write flag.
 */
enum {
    I2C_WRITE_FLAG = 0,
    I2C_READ_FLAG = 1,
};

/**
 * @brief The instances of I2C device
 */
#ifdef I2C_SFRS
I2C_State_t I2C_Device_State =
{
    .counter = 0,
    .collision = 0,
    .state = I2C_STATE_IDLE,
};

const static I2C_Device_t I2C_Device =
{
    .Enable = f_I2C_Enable,
    .Disable = f_I2C_Disable,
    .SetSlaveAddress = f_I2C_SetSlaveAddress,
    .WriteData = f_I2C_WriteData,
    .ReadData = f_I2C_ReadData,
    .GetStatus = f_I2C_GetStatus,
    .SlaveClockStretch = f_I2C_SlaveClockStretch,
    .SlaveReleaseClock = f_I2C_SlaveReleaseClock,
    .MasterRepeatStart = f_I2C_MasterRepeatStart,
    .MasterReceiveStart = f_I2C_MasterReceiveStart,
    .MasterIssueNack = f_I2C_MasterIssueNack,
    .MasterIssueAck = f_I2C_MasterIssueAck,
    .MasterIssueStop = f_I2C_MasterIssueStop,
    .MasterIssueStart = f_I2C_MasterIssueStart,
    .StartBitReceived = f_I2C_StartBitReceived,
    .Set7BitSlaveAddr = f_I2C_Set7BitSlaveAddr,
    .Set10BitSlaveAddr = f_I2C_Set10BitSlaveAddr,
    .SetMasterMode = f_I2C_InitMasterMode,
    .SetSlaveMode = f_I2C_InitSlaveMode,
    .SlaveEnableGeneralCall = f_I2C_SlaveEnableGeneralCall,
    .SlaveDisableGeneralCall = f_I2C_SlaveDisableGeneralCall,
    .EnableFilter = f_I2C_EnableFilter,
    .DisableFilter = f_I2C_DisableFilter,
    .SetSlaveAddress = f_I2C_SetSlaveAddress,
    .SetSlaveAddressMask = f_I2C_SetSlaveAddressMask,
    .SetClockDivider = f_I2C_SetClockDivider,
    .IRQ = I2C_IRQn,
    .CollisionIRQ = I2C_Collision_IRQn,
    .Name = "I2C",
    .state = &I2C_Device_State,
};    
#endif

/**
 * @brief The table of I2C device instances supported by the platform.
 */
const static I2C_Device_t *I2C_Device_Table[] = {
#ifdef I2C_SFRS
    &I2C_Device,
#endif
    NULL,
};


I2C_Device_t  *I2C_devGetDevice(const char *name)
{
    int8_t i;

    for (i = 0; i < sizeof(I2C_Device_Table)/sizeof(I2C_Device_Table[0]); i++) {
        if (strncmp(name, I2C_Device_Table[i]->Name, strlen(I2C_Device_Table[i]->Name)) == 0)
            return (I2C_Device_t  *)I2C_Device_Table[i];
    }
    return NULL;
}

int32_t I2C_devEnable(I2C_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    NVIC_EnableIRQ(dev->IRQ); 
    NVIC_EnableIRQ(dev->CollisionIRQ);
    dev->Enable();
    return 0;
}

int32_t I2C_devDisable(I2C_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    NVIC_DisableIRQ(dev->IRQ); 
    NVIC_DisableIRQ(dev->CollisionIRQ);
    dev->Disable();
    return 0;
}

int32_t I2C_devReset(I2C_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    dev->Disable();
    dev->state->state = I2C_STATE_IDLE;
    dev->state->collision = 0;
    dev->state->counter = 0;
    dev->Enable();
    return 0;
}

int32_t I2C_devSlaveInit(I2C_Device_t *dev, I2C_Device_SlaveInfo_t *info)
{
    if (!dev || !info)
        return -EFAULT;
    dev->state->mode = I2C_MODE_SLAVE;
    dev->SetSlaveMode();
    if (info->SlaveAddrLen == 10)
        dev->Set10BitSlaveAddr();
    else
        dev->Set7BitSlaveAddr();
    if (info->GeneralCallEnable)
        dev->SlaveEnableGeneralCall();
    else
        dev->SlaveDisableGeneralCall();       
    dev->SetSlaveAddress(info->SlaveAddr);
    dev->SetSlaveAddressMask(info->SlaveAddrMask);
    if (info->FilterEnable)
        dev->EnableFilter();
    else
        dev->DisableFilter();       

    I2C_devReset(dev);
    NVIC_EnableIRQ(dev->IRQ); 
    NVIC_EnableIRQ(dev->CollisionIRQ);
    
    dev->SlaveClockStretch();
    return 0;
}

int32_t I2C_devSlaveSetResponseData(I2C_Device_t *dev, uint8_t *array, uint32_t len)
{
    if (!dev)
        return -EFAULT;
    if (len > I2C_DEVICE_BUFFER_SIZE)
        return -EPERM;

    dev->state->read_len = len;
    dev->state->read_index = 0;
    memcpy(dev->state->read_buf, array, len);
    return 0;
}

int32_t I2C_devSlaveReadResult(I2C_Device_t *dev, uint8_t *array, uint32_t len)
{
    if (!dev)
        return -EFAULT;
    if (len > I2C_DEVICE_BUFFER_SIZE)
        return -EPERM;

    memcpy(array, dev->state->write_buf, len);
    return 0;
}

int32_t I2C_devSlaveDataReceived(I2C_Device_t *dev)
{
    if (!dev)
        return -EFAULT;  
    return (dev->state->write_index);
}

int32_t I2C_devSlaveDataResponsed(I2C_Device_t *dev)
{
    if (!dev)
        return -EFAULT;  
    return (dev->state->read_index);
}

int32_t I2C_devSlaveRestart(I2C_Device_t *dev)
{
    if (!dev)
        return -EFAULT;

    dev->state->counter = 0;
    dev->state->write_index = 0;
    dev->state->read_index = 0;
    dev->state->state = I2C_STATE_IDLE;

    return 0;
}

int32_t I2C_devMasterInit(I2C_Device_t  *dev, I2C_Device_MasterInfo_t *info)
{
    if (!dev || !info)
        return -EFAULT;
    dev->state->mode = I2C_MODE_MASTER;
    dev->SetMasterMode();
    dev->SetClockDivider(i2c_speed_map[info->Speed]);
    if (info->FilterEnable)
        dev->EnableFilter();
    else
        dev->DisableFilter();       
    I2C_devReset(dev);
    NVIC_EnableIRQ(dev->IRQ); 
    NVIC_EnableIRQ(dev->CollisionIRQ);
    return 0;
}

int32_t I2C_devIsInProgress(I2C_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return ((dev->state->state != I2C_STATE_IDLE) ? 1 : 0);
}

int32_t I2C_devMasterSendData(I2C_Device_t *dev, uint16_t slave_addr, uint8_t *array, uint32_t len)
{
    if (!dev)
        return -EFAULT;
    if (len > I2C_DEVICE_BUFFER_SIZE)
        return -EPERM;

    dev->state->opmode = I2C_OP_WRITE;
    dev->state->address = slave_addr;
    dev->state->counter = 0;
    dev->state->write_len = len;
    dev->state->write_index = 0;
    memcpy(dev->state->write_buf, array, len);
    dev->MasterIssueStart();
    dev->state->state = I2C_STATE_WRITE_START;
    
    return 0;
}

int32_t I2C_devMasterReadResult(I2C_Device_t *dev, uint8_t *array, uint32_t len)
{
    if (!dev)
        return -EFAULT;
    if (len > I2C_DEVICE_BUFFER_SIZE)
        return -EPERM;

    memcpy(array, dev->state->read_buf, len);
    
    return 0;
}

int32_t I2C_devMasterReadRequest(I2C_Device_t *dev, uint16_t slave_addr, uint32_t len)
{
    if (!dev)
        return -EFAULT;
    if (len > I2C_DEVICE_BUFFER_SIZE)
        return -EPERM;

    dev->state->opmode = I2C_OP_READ;
    dev->state->address = slave_addr;
    dev->state->counter = 0;
    dev->state->read_len = len;
    dev->state->read_index = 0;

    dev->MasterIssueStart();
    dev->state->state = I2C_STATE_READ_START;
    
    return 0;
}

int32_t I2C_devMasterRegisterReadRequest(I2C_Device_t *dev, uint16_t slave_addr,
    uint8_t *reg_addr, uint8_t addrlen, uint8_t datalen)
{
    if (!dev)
        return -EFAULT;
    if (datalen > I2C_DEVICE_BUFFER_SIZE)
        return -EPERM;
    if (addrlen > I2C_DEVICE_BUFFER_SIZE)
        return -EPERM;

    dev->state->opmode = I2C_OP_REGISTER_READ;
    dev->state->address = slave_addr;
    dev->state->counter = 0;
    dev->state->read_len = datalen;
    dev->state->read_index = 0;
    dev->state->write_len = addrlen;
    dev->state->write_index = 0;
    memcpy(dev->state->write_buf, reg_addr, addrlen);

    dev->MasterIssueStart();
    dev->state->state = I2C_STATE_WRITE_START;
    
    return 0;
}

//! I2C clollison interrupt handler.
void I2C_Collision_Handler ( void )
{
    I2C_Device_t *dev = (I2C_Device_t *)I2C_Device_Table[0];
    
    dev->state->status = dev->GetStatus();
    dev->state->collision++;
}

void master_isr(I2C_Device_t *dev)
{
    uint8_t data;

    switch (dev->state->state) {
    case I2C_STATE_READ_START:
        data = (dev->state->address << 1) | I2C_READ_FLAG;
        dev->WriteData(data);
        dev->state->state = I2C_STATE_RECEIVE_DATA;
        break;

    case I2C_STATE_WRITE_START:
        data = (dev->state->address << 1) | I2C_WRITE_FLAG;
        dev->WriteData(data);
        dev->state->state = I2C_STATE_WRITE_DATA;
        break;

    case I2C_STATE_REPEAT_START:
        dev->MasterRepeatStart();
        dev->state->state = I2C_STATE_READ_START;
        break;
            
    case I2C_STATE_WRITE_DATA:
        data = dev->state->write_buf[dev->state->write_index];
        dev->WriteData(data);
        dev->state->write_index++;
        if (dev->state->write_index >= dev->state->write_len) {
            if (dev->state->opmode == I2C_OP_REGISTER_READ)
                dev->state->state = I2C_STATE_REPEAT_START;
            else 
                dev->state->state = I2C_STATE_STOP;
        }
        break;

    case I2C_STATE_RECEIVE_DATA:
        dev->MasterReceiveStart();
        dev->state->state = I2C_STATE_READ_DATA;
        break;

    case I2C_STATE_READ_DATA:
        if (dev->state->read_index < dev->state->read_len) {
            dev->state->read_buf[dev->state->read_index] = dev->state->data;
            dev->state->read_index++;
        }
        if (dev->state->read_index < dev->state->read_len) {
            dev->MasterIssueAck();
            dev->state->state = I2C_STATE_RECEIVE_DATA;
        } else {
            dev->MasterIssueNack();
            dev->state->state = I2C_STATE_STOP;
        }
        break;
 
     case I2C_STATE_STOP:
        dev->MasterIssueStop();
        dev->state->state = I2C_STATE_IDLE;
        break;

    default:
        break;
    } 
}

void update_state(I2C_Device_t *dev)
{
    if (dev->state->status & E_I2C_STATUS_BIT_IADDRR) {
        if (dev->state->status & E_I2C_STATUS_BIT_IRWBUSY)
            dev->state->state = I2C_STATE_READ_DATA;
        else
            dev->state->state = I2C_STATE_WRITE_DATA;
    } else
        dev->state->state = I2C_STATE_IDLE;
}

bool is_new_session(I2C_Device_t *dev)
{
  return (dev->state->status & E_I2C_STATUS_BIT_ISTPR) |
          (dev->state->status & E_I2C_STATUS_BIT_ISTRR);
}

void slave_isr(I2C_Device_t *dev)
{     
    switch (dev->state->state) {
    case I2C_STATE_IDLE:
        if (is_new_session(dev))
            update_state(dev);
        if (dev->state->state == I2C_STATE_READ_DATA) {
            dev->WriteData(dev->state->read_buf[dev->state->read_index]);
            dev->state->read_index = (dev->state->read_index + 1)%I2C_DEVICE_BUFFER_SIZE;
        }
        break;

    case I2C_STATE_READ_DATA:
        if (is_new_session(dev)) {
            update_state(dev);
            dev->state->read_index = 0;
        } else if (dev->state->status & E_I2C_STATUS_BIT_IACKR) /* Nack received */
            dev->state->state = I2C_STATE_IDLE;
        if (dev->state->state == I2C_STATE_READ_DATA) {
            dev->WriteData(dev->state->read_buf[dev->state->read_index]);
            dev->state->read_index = (dev->state->read_index + 1)%I2C_DEVICE_BUFFER_SIZE;
        }
        break;
    
    case I2C_STATE_WRITE_DATA:
        if (is_new_session(dev)) {
            update_state(dev);
            dev->state->write_index = 0;
        } else {
            dev->state->write_buf[dev->state->write_index] = dev->state->data;
            dev->state->write_index = (dev->state->write_index + 1)%I2C_DEVICE_BUFFER_SIZE;
        }
        break;

    default:
        break;
    }

    dev->SlaveReleaseClock();
    dev->SlaveClockStretch();
}

//! I2C interrupt handler.
void I2C_Handler ( void )
{
    I2C_Device_t *dev = (I2C_Device_t *)I2C_Device_Table[0];
    
    dev->state->status = dev->GetStatus();
    dev->state->data = dev->ReadData();
    dev->state->counter++;
      
    if (dev->state->mode == I2C_MODE_SLAVE)
        slave_isr(dev);
    else
        master_isr(dev);
    
}
