/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file systick_device.c
 */

#include "systick_device.h"
#include "errno.h"

/**
 * @brief The instance of SysTick device
 */
const static SysTick_Device_t SysTick_Device =
{
    .Enable = f_SYSTICK_Enable,
    .Disable = f_SYSTICK_Disable,
    .IsEnabled = f_SYSTICK_IsEnabled,
    .IntEnable = f_SYSTICK_IntEnable,
    .IntDisable = f_SYSTICK_IntDisable,
    .SetLoad = f_SYSTICK_SetLoad,
    .GetLoad = f_SYSTICK_GetLoad,
    .SetCount = f_SYSTICK_SetCount,
    .GetCount = f_SYSTICK_GetCount,
    .IRQ = SysTick_IRQn,
};    

SysTick_Device_t *SysTick_devRequest(void)
{
    return &SysTick_Device;
}

/**
 * @brief A structure to represent the state of a Systick device
 */
typedef struct {
    uint8_t mode;
    systick_cb_func_t cb_func;
    uint64_t    counts;
} SysTick_State_t;

SysTick_State_t SysTick_State =
{
    .cb_func = NULL, .counts = 0,
};

void SysTick_Handler(void)
{
    if (SysTick_State.cb_func != NULL)
        SysTick_State.cb_func((SysTick_Device_t *)&SysTick_Device);
    SysTick_State.counts++;
}

int32_t SysTick_devDelayTicks(SysTick_Device_t *dev, uint32_t ticks)
{
    uint32_t tick_init, tick_new, tick_old;

    if (!dev)
        return -EFAULT;
    if (!dev->IsEnabled())
        return -EPERM;
    tick_init = dev->GetCount();
    tick_new = tick_init;
    tick_old = tick_new;
    while ((tick_init - tick_new) < ticks) {
        tick_new = dev->GetCount();
        if (tick_new > tick_old)
            tick_init += dev->GetLoad();
        tick_old = tick_new;
    }
    return 0;
}

int32_t SysTick_devDelayMs(SysTick_Device_t *dev, uint32_t ms)
{
    uint64_t ticks;

    if (!dev)
        return -EFAULT;
    if (!dev->IsEnabled())
        return -EPERM;
    ticks = (E_HWCFG_SYSTEM_CLOCK_KHZ * ms);
    return SysTick_devDelayTicks(dev, (uint32_t)ticks);
}

int32_t SysTick_devGetInterruptCounts(SysTick_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    if (SysTick_State.mode != SYSTICK_MODE_ENABLE_INTERRUPT)
        return -ENODEV;
    return SysTick_State.counts;
}

int64_t SysTick_devGetElapsedMs(SysTick_Device_t *dev)
{
    uint64_t ticks;
    uint64_t counts;

    if (!dev)
        return -EFAULT;
    if (SysTick_State.mode != SYSTICK_MODE_ENABLE_INTERRUPT)
        return -ENODEV;

    counts = SysTick_State.counts;
    ticks = dev->GetLoad() - dev->GetCount();
    ticks += dev->GetLoad() * counts;
   
    if (counts == SysTick_State.counts)
        return ticks/E_HWCFG_SYSTEM_CLOCK_KHZ;
    else
        return SysTick_devGetElapsedMs(dev);
}

int32_t SysTick_devInit(SysTick_Device_t *dev, uint8_t mode, uint32_t ticks, void *cb)
{
    switch(mode) {
    case SYSTICK_MODE_ENABLE_INTERRUPT:
        SysTick_State.cb_func = (systick_cb_func_t)cb;
        NVIC_EnableIRQ(dev->IRQ);
        dev->SetLoad(ticks);
        dev->SetCount(0);
        dev->IntEnable();
        dev->Enable();
        break;

    case SYSTICK_MODE_ENABLE:
        SysTick_State.cb_func = NULL;
        NVIC_DisableIRQ(dev->IRQ);
        dev->SetLoad(ticks);
        dev->SetCount(0);
        dev->IntDisable();
        dev->Enable();
        break;

    case SYSTICK_MODE_DISABLE:
    default:
        SysTick_State.cb_func = NULL;
        NVIC_DisableIRQ(dev->IRQ);
        dev->IntDisable();
        dev->Disable();
        break;
    }
    SysTick_State.mode = mode;
    SysTick_State.counts = 0;
    return 0;
}

