/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file flash_sfrs.h
 */

#ifndef __FLASH_SFRS_H__
#define __FLASH_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for FLASH.
 */
typedef struct {
    uint32_t FLADDR;
    uint32_t FLWRDT;
    uint32_t UNLBWR;
    uint32_t BWRSTRT;
    uint32_t UNLSER;
    uint32_t SERSTRT;
    uint32_t UNLBLKER;
    uint32_t BKERSTRT;
    union {
        struct {
            uint8_t RESERVED0;
            struct {
                uint8_t BWBUSY:1;
                uint8_t SEBUSY:1;
                uint8_t BEBUSY:1;
                uint8_t RESERVED1:5;
            } FLAGS;
            uint8_t RESERVED1[2];
        };
        uint32_t WORD;
    } FLSCTRL;
    uint32_t FLSCP;
} FLASH_SFRS_t;
#define FLASH_SFRS                      ((__IO FLASH_SFRS_t  *) (FLASH_CTRL_BASE))

#define	E_FLASH_CODE_PROTECT            ((uint32_t) 0xF2E11047)
#define	E_FLASH_CODE_UNPROTECT          ((uint32_t) 0x00000000)

#define	E_FLASH_BYTE_UNLOCK_PAT         ((uint32_t) 0x55555555)
#define	E_FLASH_BYTE_WRITE_PAT          ((uint32_t) 0xAAAAAAAA)
#define	E_FLASH_BLOCK_UNLOCK_PAT        ((uint32_t) 0x77777777)
#define	E_FLASH_BLOCK_ERASE_PAT         ((uint32_t) 0x88888888)
#define	E_FLASH_SECTOR_UNLOCK_PAT       ((uint32_t) 0x66666666)
#define	E_FLASH_SECTOR_ERASE_PAT        ((uint32_t) 0x99999999)

/**
* @brief Flash memory erase block function.
*
* @param address inside the block to be erased.
*/
__ramfunc static __INLINE void f_FLASH_EraseBlock(__IO uint32_t address)
{
	FLASH_SFRS->FLADDR   = address;
	FLASH_SFRS->UNLBLKER = E_FLASH_BLOCK_UNLOCK_PAT;
	FLASH_SFRS->BKERSTRT = E_FLASH_BLOCK_ERASE_PAT;
}

/**
* @brief Flash memory erase block check function.
*
* @return Block Erase Flag,  1 if still erasing, 0 otherwise
*/
__ramfunc static __INLINE  uint32_t f_FLASH_CheckErasingBlock(void)
{
    return ( FLASH_SFRS->FLSCTRL.FLAGS.BEBUSY );
}

/**
* @brief Flash memory erase sector function.
*
* @param address inside the sector to be erased.
*/
__ramfunc static __INLINE void f_FLASH_EraseSector(__IO uint32_t address)
{
	FLASH_SFRS->FLADDR   = address;
	FLASH_SFRS->UNLSER = E_FLASH_SECTOR_UNLOCK_PAT;
	FLASH_SFRS->SERSTRT = E_FLASH_SECTOR_ERASE_PAT;
}

/**
* @brief Flash memory erase sector check function.
*
* @return Sector Erase Flag,  1 if still erasing, 0 otherwise
*/
__ramfunc static __INLINE  uint32_t f_FLASH_CheckErasingSector(void)
{
    return ( FLASH_SFRS->FLSCTRL.FLAGS.SEBUSY );
}

/**
* @brief Flash memory write byte function.
*
* @param address and data (byte) to be programmed into flash.
* @param data and data (byte) to be programmed into flash.
*/
__ramfunc static __INLINE  void f_FLASH_WriteByte(__IO uint32_t address, uint8_t data)
{
	FLASH_SFRS->FLADDR  = address;
	FLASH_SFRS->UNLBWR  = E_FLASH_BYTE_UNLOCK_PAT;
	FLASH_SFRS->BWRSTRT = E_FLASH_BYTE_WRITE_PAT;
	FLASH_SFRS->FLWRDT  = data;
}

/**
* @brief Flash memory write byte check function.
*
* @return Byte write Flag, 1 if still erasing, 0 otherwise
*/
__ramfunc static __INLINE  uint32_t f_FLASH_CheckWritingByte(void)
{
    return ( FLASH_SFRS->FLSCTRL.FLAGS.BWBUSY );
}

/**
* @brief Flash memory write byte check function.
*
* @param mode The enumeration of enable or disable code protection
*/
static __INLINE  void f_FLASH_CodeProtection(uint32_t mode)
{
    FLASH_SFRS->FLSCP = mode; 
}

#endif /* __FLASH_SFRS_H__ */