/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file gpio_sfrs.h
 */

#ifndef __GPIO_SFRS_H__
#define __GPIO_SFRS_H__

#include <stdbool.h>

/**
 * @brief A structure to represent Special Function Registers for all GPIO block.
 */
typedef struct GPIO_SFRS_t {
    uint8_t     PORTA;
    uint8_t     PORTB;
    uint8_t     PORTC;
    uint8_t     PORTD;
    uint8_t     PORTE;
    uint8_t     PORTA_OutputEn;
    uint8_t     PORTB_OutputEn;
    uint8_t     PORTC_OutputEn;
    uint8_t     PORTD_OutputEn;
    uint8_t     PORTE_OutputEn;
    union {
        struct uint8_t {
            uint8_t MDIR:1;
            uint8_t MDUART:1;
            uint8_t MDI2C:1;
            uint8_t MDSPI:1;
            uint8_t UTXPOL:1;
            uint8_t UPSWAP:1;
            uint8_t I2CRT:2;
        };
        uint8_t     BYTE;
    } PIN_CONFIG;
    uint8_t     PORTE_LEDMode;
    uint8_t     PORTA_PWMMode;
    uint8_t     PORTC_PWMMode;
    uint8_t     PORTA_IntEn;
    uint8_t     PORTB_IntEn;
    uint8_t     Reserved2;
    uint8_t     PORTD_IntEn;
    uint8_t     Reserved3[98190];
    uint8_t     PORTA_PullUp;
    uint8_t     PORTA_PullDn;
    uint8_t     PORTA_ReadEn;
    uint8_t     PORTB_PullUp;
    uint8_t     PORTB_PullDn;
    uint8_t     PORTB_ReadEn;
    uint8_t     PORTC_PullUp;
    uint8_t     PORTC_PullDn;
    uint8_t     PORTC_ReadEn;
    uint8_t     PORTD_PullUp;
    uint8_t     PORTD_PullDn;
    uint8_t     PORTD_ReadEn;
    uint8_t     PORTE_PullUp;
    uint8_t     PORTE_PullDn;
    uint8_t     PORTE_ReadEn;
    uint8_t     PORTE_LEDTrim;
} GPIO_SFRS_t;

/**
 * @brief The starting address of all GPIO block.
 */
#define GPIO_SFRS ((__IO GPIO_SFRS_t *) (ASIC_7B_BASE + 0x60))

/**
 * @brief A structure to represent Special Function Registers for GPIOA block.
 */
typedef struct {
    uint8_t     PORT;
    uint8_t     RESERVED0[4];
    uint8_t     OUTPUTEN;
    uint8_t     RESERVED1[6];
    uint8_t     PWMMODE;
    uint8_t     RESERVED2;
    uint8_t     INTEN;
    uint8_t     RESERVED3[98193];
    uint8_t     PULLUP;
    uint8_t     PULLDOWN;
    uint8_t     READEN;
} GPIOA_SFRS_t;

/**
 * @brief The starting address of GPIOA block.
 */
#define GPIOA_SFRS ((__IO GPIOA_SFRS_t *)(ASIC_7B_BASE + 0x60))

/**
 * @brief Enable/disable pull up on GPIOA
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOA_EnablePullUp(uint8_t pos, bool bit)
{
    if (bit)
        GPIOA_SFRS->PULLUP &=  ~(1 << pos);
    else
        GPIOA_SFRS->PULLUP |=  (1 << pos);
}

/**
 * @brief Enable/disable pull down on GPIOA
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOA_EnablePullDown(uint8_t pos, bool bit)
{
    if (bit)
        GPIOA_SFRS->PULLDOWN |=  (1 << pos);
    else
        GPIOA_SFRS->PULLDOWN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable output on GPIOA
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOA_EnableOutput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOA_SFRS->OUTPUTEN |=  (1 << pos);
    else
        GPIOA_SFRS->OUTPUTEN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable input on GPIOA
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOA_EnableInput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOA_SFRS->READEN |=  (1 << pos);
    else
        GPIOA_SFRS->READEN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable interrupt on GPIOA
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOA_EnableInterrupt(uint8_t pos, bool bit)
{
    if (bit)
        GPIOA_SFRS->INTEN |=  (1 << pos);
    else
        GPIOA_SFRS->INTEN &=  ~(1 << pos);
}

/**
 * @brief Read pin state on GPIOA
 *
 * @param pos The pin position.
 * @return The pin state.
 */
static __INLINE uint8_t GPIOA_Read(uint8_t pos)
{
    return GPIOA_SFRS->PORT >> pos & 1;
}

/**
 * @brief Set pin state on GPIOA
 *
 * @param pos The pin position.
 * @param bit The pin state.
 */
static __INLINE void GPIOA_Set(uint8_t pos, uint8_t bit)
{
    uint8_t readen;
    
    readen = GPIOA_SFRS->READEN;
    GPIOA_SFRS->READEN = 0xFF;
    if (bit)
        GPIOA_SFRS->PORT |=  (1 << pos);
    else
        GPIOA_SFRS->PORT &=  ~(1 << pos);
    GPIOA_SFRS->READEN = readen;
}

/**
 * @brief Enable/disable pull up bits on GPIOA
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOA_EnablePortPullUp(bool bit, uint8_t mask)
{
    if (bit)
        GPIOA_SFRS->PULLUP &= ~mask;
    else
        GPIOA_SFRS->PULLUP |= mask;
}

/**
 * @brief Enable/disable pull down on GPIOA
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOA_EnablePortPullDown(bool bit, uint8_t mask)
{
    if (bit)
        GPIOA_SFRS->PULLDOWN |= mask;
    else
        GPIOA_SFRS->PULLDOWN &= ~mask;
}

/**
 * @brief Enable/disable output enable bits on GPIOA
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOA_EnablePortOutput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOA_SFRS->OUTPUTEN |=  mask;
    else
        GPIOA_SFRS->OUTPUTEN &= ~mask;
}

/**
 * @brief Enable/disable input enable bits on GPIOA
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOA_EnablePortInput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOA_SFRS->READEN |= mask;
    else
        GPIOA_SFRS->READEN &= ~mask;
}

/**
 * @brief Enable/disable interrupt enable bits on GPIOA
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOA_EnablePortInterrupt(bool bit, uint8_t mask)
{
    if (bit)
        GPIOA_SFRS->INTEN |= mask;
    else
        GPIOA_SFRS->INTEN &= ~mask;
}

/**
 * @brief Set port value on GPIOA
 *
 * @param val The value to be set to GPIO port.
 * @param mask The bit mask for the pins to be configured.
 * @return None.
 */
static __INLINE void GPIOA_PortSet(uint8_t val, uint8_t mask)
{
    uint8_t readen;
    
    readen = GPIOA_SFRS->READEN;
    GPIOA_SFRS->READEN = 0xFF;
    GPIOA_SFRS->PORT &= ~mask;
    GPIOA_SFRS->PORT |= (val & mask);
    GPIOA_SFRS->READEN = readen;
}

/**
 * @brief Read port value from GPIOA
 *
 * @param mask The bit mask for the pins to be read.
 * @return The masked port value from GPIOA.
 */
static __INLINE uint8_t GPIOA_PortRead(uint8_t mask)
{
    return GPIOA_SFRS->PORT & mask;
}

/**
 * @brief A structure to represent Special Function Registers for GPIOB.
 */
typedef struct {
    uint8_t     PORT;
    uint8_t     RESERVED0[4];
    uint8_t     OUTPUTEN;
    uint8_t     RESERVED1[8];
    uint8_t     INTEN;
    uint8_t     RESERVED2[98195];
    uint8_t     PULLUP;
    uint8_t     PULLDOWN;
    uint8_t     READEN;
} GPIOB_SFRS_t;

/**
 * @brief The starting address of GPIOB block.
 */
#define GPIOB_SFRS ((__IO GPIOB_SFRS_t *) (ASIC_7B_BASE + 0x61))

/**
 * @brief Enable/disable pull up on GPIOB
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOB_EnablePullUp(uint8_t pos, bool bit)
{
    if (bit)
        GPIOB_SFRS->PULLUP &=  ~(1 << pos);
    else
        GPIOB_SFRS->PULLUP |=  (1 << pos);
}

/**
 * @brief Enable/disable pull down on GPIOB
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOB_EnablePullDown(uint8_t pos, bool bit)
{
    if (bit)
        GPIOB_SFRS->PULLDOWN |=  (1 << pos);
    else
        GPIOB_SFRS->PULLDOWN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable output on GPIOB
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOB_EnableOutput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOB_SFRS->OUTPUTEN |=  (1 << pos);
    else
        GPIOB_SFRS->OUTPUTEN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable Input on GPIOB
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOB_EnableInput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOB_SFRS->READEN |=  (1 << pos);
    else
        GPIOB_SFRS->READEN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable interrupt on GPIOA
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOB_EnableInterrupt(uint8_t pos, bool bit)
{
    if (bit)
        GPIOB_SFRS->INTEN |=  (1 << pos);
    else
        GPIOB_SFRS->INTEN &=  ~(1 << pos);
}

/**
 * @brief Read pin state on GPIOB
 *
 * @param pos The pin position.
 * @return The pin state.
 */
static __INLINE uint8_t GPIOB_Read(uint8_t pos)
{
    return GPIOB_SFRS->PORT >> pos & 1;
}

/**
 * @brief Set pin state on GPIOB
 *
 * @param pos The pin position.
 * @param bit The pin state.
 */
static __INLINE void GPIOB_Set(uint8_t pos, uint8_t bit)
{
    uint8_t readen;
    
    readen = GPIOB_SFRS->READEN;
    GPIOB_SFRS->READEN = 0xFF;
    if (bit)
        GPIOB_SFRS->PORT |=  (1 << pos);
    else
        GPIOB_SFRS->PORT &=  ~(1 << pos);
    GPIOB_SFRS->READEN = readen;   
}

/**
 * @brief Enable/disable pull up bits on GPIOB
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOB_EnablePortPullUp(bool bit, uint8_t mask)
{
    if (bit)
        GPIOB_SFRS->PULLUP &= ~mask;
    else
        GPIOB_SFRS->PULLUP |= mask;
}

/**
 * @brief Enable/disable pull down enable bits on GPIOB
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOB_EnablePortPullDown(bool bit, uint8_t mask)
{
    if (bit)
        GPIOB_SFRS->PULLDOWN |= mask;
    else
        GPIOB_SFRS->PULLDOWN &= ~mask;
}

/**
 * @brief Enable/disable output enable bits on GPIOB
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOB_EnablePortOutput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOB_SFRS->OUTPUTEN |=  mask;
    else
        GPIOB_SFRS->OUTPUTEN &= ~mask;
}

/**
 * @brief Enable/disable input enable bits on GPIOB
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOB_EnablePortInput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOB_SFRS->READEN |= mask;
    else
        GPIOB_SFRS->READEN &= ~mask;
}

/**
 * @brief Enable/disable interrupt enable bits on GPIOB
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOB_EnablePortInterrupt(bool bit, uint8_t mask)
{
    if (bit)
        GPIOB_SFRS->INTEN |= mask;
    else
        GPIOB_SFRS->INTEN &= ~mask;
}

/**
 * @brief Set port value on GPIOB
 *
 * @param val The value to be set to GPIO port.
 * @param mask The bit mask for the pins to be configured.
 * @return None.
 */
static __INLINE void GPIOB_PortSet(uint8_t val, uint8_t mask)
{
    uint8_t readen;
    
    readen = GPIOB_SFRS->READEN;
    GPIOB_SFRS->READEN = 0xFF;
    GPIOB_SFRS->PORT &= ~mask;
    GPIOB_SFRS->PORT |= (val & mask);
    GPIOB_SFRS->READEN = readen;
}

/**
 * @brief Read port value from GPIOB
 *
 * @param mask The bit mask for the pins to be read.
 * @return The masked port value from GPIOB.
 */
static __INLINE uint8_t GPIOB_PortRead(uint8_t mask)
{
    return GPIOB_SFRS->PORT & mask;
}

/**
 * @brief A structure to represent Special Function Registers for GPIOC.
 */
typedef struct {
    uint8_t     PORT;
    uint8_t     RESERVED0[4];
    uint8_t     OUTPUTEN;
    uint8_t     RESERVED1[5];
    uint8_t     PWMMODE;
    uint8_t     RESERVED2[98200];
    uint8_t     PULLUP;
    uint8_t     PULLDOWN;
    uint8_t     READEN;
} GPIOC_SFRS_t;

/**
 * @brief The starting address of GPIOC block.
 */
#define GPIOC_SFRS ((__IO GPIOC_SFRS_t *) (ASIC_7B_BASE + 0x62))

/**
 * @brief Enable/disable pull up on GPIOC
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOC_EnablePullUp(uint8_t pos, bool bit)
{
    if (bit)
        GPIOC_SFRS->PULLUP &=  ~(1 << pos);
    else
        GPIOC_SFRS->PULLUP |=  (1 << pos);
}

/**
 * @brief Enable/disable pull down on GPIOC
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOC_EnablePullDown(uint8_t pos, bool bit)
{
    if (bit)
        GPIOC_SFRS->PULLDOWN |=  (1 << pos);
    else
        GPIOC_SFRS->PULLDOWN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable output on GPIOC
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOC_EnableOutput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOC_SFRS->OUTPUTEN |=  (1 << pos);
    else
        GPIOC_SFRS->OUTPUTEN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable Input on GPIOC
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOC_EnableInput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOC_SFRS->READEN |=  (1 << pos);
    else
        GPIOC_SFRS->READEN &=  ~(1 << pos);
}

/**
 * @brief Read pin state on GPIOC
 *
 * @param pos The pin position.
 * @return The pin state.
 */
static __INLINE uint8_t GPIOC_Read(uint8_t pos)
{
    return GPIOC_SFRS->PORT >> pos & 1;
}

/**
 * @brief Set pin state on GPIOC
 *
 * @param pos The pin position.
 * @param bit The pin state.
 */
static __INLINE void GPIOC_Set(uint8_t pos, uint8_t bit)
{
    uint8_t readen;
    
    readen = GPIOC_SFRS->READEN;
    GPIOC_SFRS->READEN = 0xFF;
    if (bit)
        GPIOC_SFRS->PORT |=  (1 << pos);
    else
        GPIOC_SFRS->PORT &=  ~(1 << pos);
    GPIOC_SFRS->READEN = readen;
}

/**
 * @brief Enable/disable pull up bits on GPIOC
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOC_EnablePortPullUp(bool bit, uint8_t mask)
{
    if (bit)
        GPIOC_SFRS->PULLUP &= ~mask;
    else
        GPIOC_SFRS->PULLUP |= mask;
}

/**
 * @brief Enable/disable pull down on GPIOC
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOC_EnablePortPullDown(bool bit, uint8_t mask)
{
    if (bit)
        GPIOC_SFRS->PULLDOWN |= mask;
    else
        GPIOC_SFRS->PULLDOWN &= ~mask;
}

/**
 * @brief Enable/disable output enable bits on GPIOC
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOC_EnablePortOutput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOC_SFRS->OUTPUTEN |=  mask;
    else
        GPIOC_SFRS->OUTPUTEN &= ~mask;
}

/**
 * @brief Enable/disable input enable bits on GPIOC
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOC_EnablePortInput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOC_SFRS->READEN |= mask;
    else
        GPIOC_SFRS->READEN &= ~mask;
}

/**
 * @brief Set port value on GPIOC
 *
 * @param val The value to be set to GPIO port.
 * @param mask The bit mask for the pins to be configured.
 * @return None.
 */
static __INLINE void GPIOC_PortSet(uint8_t val, uint8_t mask)
{
    uint8_t readen;
    
    readen = GPIOC_SFRS->READEN;
    GPIOC_SFRS->READEN = 0xFF;
    GPIOC_SFRS->PORT &= ~mask;
    GPIOC_SFRS->PORT |= (val & mask);
    GPIOC_SFRS->READEN = readen;
}

/**
 * @brief Read port value from GPIOC
 *
 * @param mask The bit mask for the pins to be read.
 * @return The masked port value from GPIOC.
 */
static __INLINE uint8_t GPIOC_PortRead(uint8_t mask)
{
    return GPIOC_SFRS->PORT & mask;
}

/**
 * @brief A structure to represent Special Function Registers for GPIOD.
 */
typedef struct {
    uint8_t     PORT;
    uint8_t     RESERVED0[4];
    uint8_t     OUTPUTEN;
    uint8_t     RESERVED1[8];
    uint8_t     INTEN;
    uint8_t     RESERVED2[98199];
    uint8_t     PULLUP;
    uint8_t     PULLDOWN;
    uint8_t     READEN;
} GPIOD_SFRS_t;

/**
 * @brief The starting address of GPIOD block.
 */
#define GPIOD_SFRS ((__IO GPIOD_SFRS_t *) (ASIC_7B_BASE + 0x63))

/**
 * @brief Enable/disable pull up on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOD_EnablePullUp(uint8_t pos, bool bit)
{
    if (bit)
        GPIOD_SFRS->PULLUP &=  ~(1 << pos);
    else
        GPIOD_SFRS->PULLUP |=  (1 << pos);
}

/**
 * @brief Enable/disable pull down on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOD_EnablePullDown(uint8_t pos, bool bit)
{
    if (bit)
        GPIOD_SFRS->PULLDOWN |=  (1 << pos);
    else
        GPIOD_SFRS->PULLDOWN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable output on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOD_EnableOutput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOD_SFRS->OUTPUTEN |=  (1 << pos);
    else
        GPIOD_SFRS->OUTPUTEN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable Input on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOD_EnableInput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOD_SFRS->READEN |=  (1 << pos);
    else
        GPIOD_SFRS->READEN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable interrupt on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOD_EnableInterrupt(uint8_t pos, bool bit)
{
    if (bit)
        GPIOD_SFRS->INTEN |=  (1 << pos);
    else
        GPIOD_SFRS->INTEN &=  ~(1 << pos);
}

/**
 * @brief Read pin state on GPIOD
 *
 * @param pos The pin position.
 * @return The pin state.
 */
static __INLINE uint8_t GPIOD_Read(uint8_t pos)
{
    return GPIOD_SFRS->PORT >> pos & 1;
}

/**
 * @brief Set pin state on GPIOD
 *
 * @param pos The pin position.
 * @param bit The pin state.
 */
static __INLINE void GPIOD_Set(uint8_t pos, uint8_t bit)
{
    uint8_t readen;
    
    readen = GPIOD_SFRS->READEN;
    GPIOD_SFRS->READEN = 0xFF;
    if (bit)
        GPIOD_SFRS->PORT |=  (1 << pos);
    else
        GPIOD_SFRS->PORT &=  ~(1 << pos);
    GPIOD_SFRS->READEN = readen;
}

/**
 * @brief Enable/disable pull up bits on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOD_EnablePortPullUp(bool bit, uint8_t mask)
{
    if (bit)
        GPIOD_SFRS->PULLUP &= ~mask;
    else
        GPIOD_SFRS->PULLUP |= mask;
}

/**
 * @brief Enable/disable pull down on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOD_EnablePortPullDown(bool bit, uint8_t mask)
{
    if (bit)
        GPIOD_SFRS->PULLDOWN |= mask;
    else
        GPIOD_SFRS->PULLDOWN &= ~mask;
}

/**
 * @brief Enable/disable output enable bits on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOD_EnablePortOutput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOD_SFRS->OUTPUTEN |=  mask;
    else
        GPIOD_SFRS->OUTPUTEN &= ~mask;
}

/**
 * @brief Enable/disable input enable bits on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOD_EnablePortInput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOD_SFRS->READEN |= mask;
    else
        GPIOD_SFRS->READEN &= ~mask;
}

/**
 * @brief Enable/disable interrupt enable bits on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOD_EnablePortInterrupt(bool bit, uint8_t mask)
{
    if (bit)
        GPIOD_SFRS->INTEN |= mask;
    else
        GPIOD_SFRS->INTEN &= ~mask;
}

/**
 * @brief Set port value on GPIOD
 *
 * @param val The value to be set to GPIO port.
 * @param mask The bit mask for the pins to be configured.
 * @return None.
 */
static __INLINE void GPIOD_PortSet(uint8_t val, uint8_t mask)
{
    uint8_t readen;
    
    readen = GPIOD_SFRS->READEN;
    GPIOD_SFRS->READEN = 0xFF;
    GPIOD_SFRS->PORT &= ~mask;
    GPIOD_SFRS->PORT |= (val & mask);
    GPIOD_SFRS->READEN = readen;
}

/**
 * @brief Read port value from GPIOD
 *
 * @param mask The bit mask for the pins to be read.
 * @return The masked port value from GPIOD.
 */
static __INLINE uint8_t GPIOD_PortRead(uint8_t mask)
{
    return GPIOD_SFRS->PORT & mask;
}

/**
 * @brief A structure to represent Special Function Registers for GPIOE.
 */
typedef struct {
    uint8_t     PORT;
    uint8_t     RESERVED0[4];
    uint8_t     OUTPUTEN;
    uint8_t     RESERVED1;
    uint8_t     LEDMODE;
    uint8_t     RESERVED2[98208];
    uint8_t     PULLUP;
    uint8_t     PULLDOWN;
    uint8_t     READEN;
} GPIOE_SFRS_t;

/**
 * @brief The starting address of GPIOE block.
 */
#define GPIOE_SFRS ((__IO GPIOE_SFRS_t *) (ASIC_7B_BASE + 0x64))

/**
 * @brief Enable/disable pull up on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOE_EnablePullUp(uint8_t pos, bool bit)
{
    if (bit)
        GPIOE_SFRS->PULLUP &=  ~(1 << pos);
    else
        GPIOE_SFRS->PULLUP |=  (1 << pos);
}

/**
 * @brief Enable/disable pull down on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOE_EnablePullDown(uint8_t pos, bool bit)
{
    if (bit)
        GPIOE_SFRS->PULLDOWN |=  (1 << pos);
    else
        GPIOE_SFRS->PULLDOWN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable output on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOE_EnableOutput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOE_SFRS->OUTPUTEN |=  (1 << pos);
    else
        GPIOE_SFRS->OUTPUTEN &=  ~(1 << pos);
}

/**
 * @brief Enable/disable Input on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
static __INLINE void GPIOE_EnableInput(uint8_t pos, bool bit)
{
    if (bit)
        GPIOE_SFRS->READEN |=  (1 << pos);
    else
        GPIOE_SFRS->READEN &=  ~(1 << pos);
}

/**
 * @brief Read pin state on GPIOE
 *
 * @param pos The pin position.
 * @return The pin state.
 */
static __INLINE uint8_t GPIOE_Read(uint8_t pos)
{
    return GPIOE_SFRS->PORT >> pos & 1;
}

/**
 * @brief Set pin state on GPIOE
 *
 * @param pos The pin position.
 * @param bit The pin state.
 */
static __INLINE void GPIOE_Set(uint8_t pos, uint8_t bit)
{
    uint8_t readen;
    
    readen = GPIOE_SFRS->READEN;
    GPIOE_SFRS->READEN = 0xFF;
    if (bit)
        GPIOE_SFRS->PORT |=  (1 << pos);
    else
        GPIOE_SFRS->PORT &=  ~(1 << pos);
    GPIOE_SFRS->READEN = readen;
}

/**
 * @brief Enable/disable pull up bits on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOE_EnablePortPullUp(bool bit, uint8_t mask)
{
    if (bit)
        GPIOE_SFRS->PULLUP &= ~mask;
    else
        GPIOE_SFRS->PULLUP |= mask;
}

/**
 * @brief Enable/disable pull down bits on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOE_EnablePortPullDown(bool bit, uint8_t mask)
{
    if (bit)
        GPIOE_SFRS->PULLDOWN |= mask;
    else
        GPIOE_SFRS->PULLDOWN &= ~mask;
}

/**
 * @brief Enable/disable output enable bits on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOE_EnablePortOutput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOE_SFRS->OUTPUTEN |=  mask;
    else
        GPIOE_SFRS->OUTPUTEN &= ~mask;
}

/**
 * @brief Enable/disable input enable bits on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
static __INLINE void GPIOE_EnablePortInput(bool bit, uint8_t mask)
{
    if (bit)
        GPIOE_SFRS->READEN |= mask;
    else
        GPIOE_SFRS->READEN &= ~mask;
}

/**
 * @brief Set port value on GPIOE
 *
 * @param val The value to be set to GPIO port.
 * @param mask The bit mask for the pins to be configured.
 * @return None.
 */
static __INLINE void GPIOE_PortSet(uint8_t val, uint8_t mask)
{
    uint8_t readen;
    
    readen = GPIOE_SFRS->READEN;
    GPIOE_SFRS->READEN = 0xFF;
    GPIOE_SFRS->PORT &= ~mask;
    GPIOE_SFRS->PORT |= (val & mask);
    GPIOE_SFRS->READEN = readen;
}

/**
 * @brief Read port value from GPIOE
 *
 * @param mask The bit mask for the pins to be read.
 * @return The masked port value from GPIOE.
 */
static __INLINE uint8_t GPIOE_PortRead(uint8_t mask)
{
    return GPIOE_SFRS->PORT & mask;
}

#define E_GPIO_I2C_PULLIP_OPEN  0
#define E_GPIO_I2C_PULLIP_1K    1
#define E_GPIO_I2C_PULLIP_10K   2
#define E_GPIO_I2C_PULLIP_100K  3

#define E_GPIO_PIN_MODE_OFF     0
#define E_GPIO_PIN_MODE_ON      1

#define E_GPIO_PIN_NOINVERT     0
#define E_GPIO_PIN_INVERT       1

#define E_GPIO_PIN_NOSWAP       0
#define E_GPIO_PIN_SWAP         1

/**
 * @brief Config GPIO pin to I2C mode
 *
 * @param mode GPIO mode or non GPIO mode (I2C mode).
 * @param pu The pull up settings.
 */
static __INLINE void GPIO_ConfigI2C(uint8_t mode, uint8_t pu)
{
    GPIO_SFRS->PIN_CONFIG.I2CRT = pu;
    GPIO_SFRS->PIN_CONFIG.MDI2C = mode;
}

/**
 * @brief Config GPIO pin to UART mode
 *
 * @param mode GPIO mode or non GPIO mode (UART mode).
 * @param swap The pin swap setting.
 * @param invert The polarity setting.
 */
static __INLINE void GPIO_ConfigUART(uint8_t mode, uint8_t swap, uint8_t invert)
{
    GPIO_SFRS->PIN_CONFIG.MDUART = mode;
    GPIO_SFRS->PIN_CONFIG.UPSWAP = swap;
    GPIO_SFRS->PIN_CONFIG.UTXPOL = invert;
}

/**
 * @brief Config GPIO pin to SPI mode
 *
 * @param mode GPIO mode or non GPIO mode (SPI mode).
 */
static __INLINE void GPIO_ConfigSPI(uint8_t mode)
{
    GPIO_SFRS->PIN_CONFIG.MDSPI = mode;
}

/**
 * @brief Config GPIO pin to PWM1
 *
 * @param pos GPIO pin position.
 */
static __INLINE void GPIO_ConfigSetModePWM1(uint8_t pos)
{
    GPIOA_SFRS->PWMMODE |= (1 << pos);      
}

/**
 * @brief Set PWM1 back to GPIO mode
 *
 * @param pos GPIO pin position.
 */
static __INLINE void GPIO_ConfigClrModePWM1(uint8_t pos)
{
    GPIOA_SFRS->PWMMODE &= ~(1 << pos);
}

/**
 * @brief Config GPIO pin to PWM2
 *
 * @param pos GPIO pin position.
 */
static __INLINE void GPIO_ConfigSetModePWM2(uint8_t pos)
{
    GPIOC_SFRS->PWMMODE |= (1 << pos);      
}

/**
 * @brief Set PWM2 back to GPIO mode
 *
 * @param pos GPIO pin position.
 */
static __INLINE void GPIO_ConfigClrModePWM2(uint8_t pos)
{
    GPIOC_SFRS->PWMMODE &= ~(1 << pos);
}

/**
 * @brief Set GPIO to LED pin
 *
 * @param pos GPIO pin position.
 */
static __INLINE void GPIO_ConfigSetModeLED(uint8_t pos)
{
    GPIOE_SFRS->LEDMODE |= (1 << pos);      
}

/**
 * @brief Set LED pin back to GPIO
 *
 * @param pos GPIO pin position.
 */
static __INLINE void GPIO_ConfigClrModeLED(uint8_t pos)
{
    GPIOE_SFRS->LEDMODE &= ~(1 << pos);
}

#endif /* __GPIO_SFRS_H__ */