/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file i2c_sfrs.h
 */

#ifndef __I2C_SFRS_H__
#define __I2C_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for I2C.
 */
typedef struct {
    union {
        struct {
            uint8_t IRBUFOVL:1;
            uint8_t IWBUFOVL:1;
            uint8_t IBUFF:1;
            uint8_t IRWBUSY:1;
            uint8_t ISTPR:1;
            uint8_t ISTRR:1;
            uint8_t IADDRR:1;
            uint8_t IACKR:1;
        };
        uint8_t BYTE;    
    } STATUS;
    union {
        struct {
            uint8_t ISTRSTRETCH:1;
            uint8_t ISTPSIZE:1;
            uint8_t ISACK:1;
            uint8_t ISNACK:1;
            uint8_t IRCSTRT:1;
            uint8_t IGC:1;
            uint8_t ICLKSTR:1;
            uint8_t IRSTR:1;
        };
        uint8_t BYTE;
    } CTRL1;
    union {
        struct {
            uint8_t IMS:1;
            uint8_t IFILTER:1;
            uint8_t IEN:1;
            uint8_t IMSK:5;
        };
        uint8_t BYTE;
    } CTRL2;
    uint8_t  DATA;
    uint8_t  IADDR0;
    uint8_t  IADDR1;
} I2C_SFRS_t;

/**
 * @brief The starting address of I2C SFRS.
 */
#define I2C_SFRS              ((__IO I2C_SFRS_t *) (SDIO_BASE + 0x08))

/**
 * @brief The enumeration of I2C status.
 */
#define E_I2C_STATUS_BIT_IRBUFOVL     ((uint8_t) (1<<0))     /*!< Read buffer overflow */
#define E_I2C_STATUS_BIT_IWBUFOVL     ((uint8_t) (1<<1))     /*!< Write buffer overflow */
#define E_I2C_STATUS_BIT_IBUFF        ((uint8_t) (1<<2))     /*!< Buffer full */
#define E_I2C_STATUS_BIT_IRWBUSY      ((uint8_t) (1<<3))     /*!< Read or write busy */
#define E_I2C_STATUS_BIT_ISTPR        ((uint8_t) (1<<4))     /*!< Stop bit received in slave mode */
#define E_I2C_STATUS_BIT_ISTRR        ((uint8_t) (1<<5))     /*!< Start bit received */
#define E_I2C_STATUS_BIT_IADDRR       ((uint8_t) (1<<6))     /*!< Data or address received in slave mode */
#define E_I2C_STATUS_BIT_IACKR        ((uint8_t) (1<<7))     /*!< Ack Received */

/**
 * @brief The enumeration of I2C control 1 register.
 */
#define E_I2C_CTRL1_BIT_ISTRSTRETCH     ((uint8_t) (1<<0))     /*!< Start clock stretch */
#define E_I2C_CTRL1_BIT_ISTPSIZE        ((uint8_t) (1<<1))     /*!< Issue a Stop bit */
#define E_I2C_CTRL1_BIT_ISACK           ((uint8_t) (1<<2))     /*!< Issue Ack */
#define E_I2C_CTRL1_BIT_ISNACK          ((uint8_t) (1<<3))     /*!< Set Ack bit value */
#define E_I2C_CTRL1_BIT_IRCSTRT         ((uint8_t) (1<<4))     /*!< Start receiving */
#define E_I2C_CTRL1_BIT_IGC             ((uint8_t) (1<<5))     /*!< Enable general call */
#define E_I2C_CTRL1_BIT_ICLKSTR         ((uint8_t) (1<<6))     /*!< Release clock stretch */
#define E_I2C_CTRL1_BIT_IRSTR           ((uint8_t) (1<<7))     /*!< Send Repeat Start bit */

/**
 * @brief Enable I2C Module
 */
static __INLINE void f_I2C_Enable(void)
{
    I2C_SFRS->CTRL2.IEN = 1;
}

/**
 * @brief Disable I2C Module
 */
static __INLINE void f_I2C_Disable(void)
{
    I2C_SFRS->CTRL2.IEN = 0;
}

/**
 * @brief Write a byte to I2C module
 */
static __INLINE void f_I2C_WriteData(uint8_t data)
{
    I2C_SFRS->DATA = data;
}

/**
 * @brief Read a byte from I2C module
 */
static __INLINE uint8_t f_I2C_ReadData(void)
{
    return I2C_SFRS->DATA;
}

/**
 * @brief Get the status of I2C module
 *
 * @return The value of status register of I2C module.
 */
static __INLINE uint8_t f_I2C_GetStatus(void)
{
    return I2C_SFRS->STATUS.BYTE;
}

/**
 * @brief Start clock stretch in I2C slave mode
 */
static __INLINE void f_I2C_SlaveClockStretch(void)
{
    I2C_SFRS->CTRL1.ISTRSTRETCH = 1;
}

/**
 * @brief Release clock stretch in I2C slave mode
 */
static __INLINE void f_I2C_SlaveReleaseClock(void)
{
    I2C_SFRS->CTRL1.ICLKSTR = 1;
}

/**
 * @brief Enable general call mode in I2C slave mode
 */
static __INLINE void f_I2C_SlaveEnableGeneralCall(void)
{
    I2C_SFRS->CTRL1.IGC = 1;
}

/**
 * @brief Disable general call mode in I2C slave mode
 */
static __INLINE void f_I2C_SlaveDisableGeneralCall(void)
{
    I2C_SFRS->CTRL1.IGC = 0;
}

/**
 * @brief Set slave address length
 */
static __INLINE void f_I2C_SlaveAddreeSizeTen(void)
{
    I2C_SFRS->CTRL1.ISTPSIZE = 1;
}

/**
 * @brief Send Repeat Start bit in I2C master mode
 */
static __INLINE void f_I2C_MasterRepeatStart(void)
{
    I2C_SFRS->CTRL1.IRSTR = 1;
}

/**
 * @brief Start to receive data in I2C master mode
 */
static __INLINE void f_I2C_MasterReceiveStart(void)
{
    I2C_SFRS->CTRL1.IRCSTRT = 1;
}

/**
 * @brief Issue NACK bit in I2C master mode
 */
static __INLINE void f_I2C_MasterIssueNack(void)
{
    uint8_t ctrl = I2C_SFRS->CTRL1.BYTE;

    ctrl |= E_I2C_CTRL1_BIT_ISNACK;
    ctrl |= E_I2C_CTRL1_BIT_ISACK;
    I2C_SFRS->CTRL1.BYTE = ctrl;
}

/**
 * @brief Issue Ack bit in I2C master mode
 */
static __INLINE void f_I2C_MasterIssueAck(void)
{
    uint8_t ctrl = I2C_SFRS->CTRL1.BYTE;

    ctrl &= ~E_I2C_CTRL1_BIT_ISNACK;
    ctrl |= E_I2C_CTRL1_BIT_ISACK;
    I2C_SFRS->CTRL1.BYTE = ctrl;
}

/**
 * @brief Issue a Stop bit in I2C master mode
 */
static __INLINE void f_I2C_MasterIssueStop(void)
{
    I2C_SFRS->CTRL1.ISTPSIZE = 1;
}

/**
 * @brief Issue a Start bit in I2C master mode
 */
static __INLINE void f_I2C_MasterIssueStart(void)
{
    I2C_SFRS->CTRL1.ISTRSTRETCH = 1;
}

/**
 * @brief Initialize I2C module to master mode
 */
static __INLINE void f_I2C_InitMasterMode(void)
{
    I2C_SFRS->CTRL2.IMS = 1;
}

/**
 * @brief Initialize I2C module to slave mode
 */
static __INLINE void f_I2C_InitSlaveMode(void)
{
    I2C_SFRS->CTRL2.IMS = 0;
}

/**
 * @brief Check if I2C module a master mode
 */
static __INLINE uint8_t f_I2C_IsMasterMode(void)
{
    return I2C_SFRS->CTRL2.IMS;
}

/**
 * @brief Set I2C slave address in 7-bit mode
 */
static __INLINE void f_I2C_Set7BitSlaveAddr(void)
{
    I2C_SFRS->CTRL1.ISTPSIZE = 0;
}

/**
 * @brief Set I2C slave address in 10-bit mode
 */
static __INLINE void f_I2C_Set10BitSlaveAddr(void)
{
    I2C_SFRS->CTRL1.ISTPSIZE = 1;
}

/**
 * @brief Enable I2C filter
 */
static __INLINE void f_I2C_EnableFilter(void)
{
    I2C_SFRS->CTRL2.IFILTER = 1;
}

/**
 * @brief Disable I2C filter
 */
static __INLINE void f_I2C_DisableFilter(void)
{
    I2C_SFRS->CTRL2.IFILTER = 0;
}

/**
 * @brief Set I2C address in slave mode
 *
 * @param addr The slave address.
 */
static __INLINE void f_I2C_SetSlaveAddress(uint16_t addr)
{
    I2C_SFRS->IADDR0 = addr & 0xFF;
    I2C_SFRS->IADDR1 = (addr >> 8) & 0x03;
}

/**
 * @brief Set I2C address mask in slave mode
 *
 * @param mask The slave address mask.
 */
static __INLINE void f_I2C_SetSlaveAddressMask(uint16_t mask)
{
    I2C_SFRS->CTRL2.IMSK = mask;
}

/**
 * @brief Set clock divider for I2C module
 *
 * @param div The divider for I2C clock.
 */
static __INLINE void f_I2C_SetClockDivider(uint8_t div)
{
    I2C_SFRS->IADDR0 = div;
}

/**
 * @brief Check if I2C write buffer full
 *
 * @return The write buffer full bit of the I2C status register.
 */
static __INLINE uint8_t f_I2C_WriteBuffNotEmpty(void)
{
    return I2C_SFRS->STATUS.IWBUFOVL;
}

/**
 * @brief Check if I2C buffer full
 *
 * @return The buffer full bit of the I2C status register.
 */
static __INLINE uint8_t f_I2C_BuffFull(void)
{
    return I2C_SFRS->STATUS.IBUFF;
}

/**
 * @brief Check if I2C Start bit is received.
 *
 * @return The state of Start Bit Received in I2C status register.
 */
static __INLINE uint8_t f_I2C_StartBitReceived(void)
{
    return I2C_SFRS->STATUS.ISTRR;
}
#endif /* __I2C_SFRS_H__ */

