/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file pwm_sfrs.h
 */
 
#ifndef __PWM_SFRS_H__
#define __PWM_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for PWM1.
 */
typedef struct {
    union {
        struct {
            uint8_t PRESC:3;
            uint8_t Reserved1:1;
            uint8_t INV:1;
            uint8_t Reserved2:2;
            uint8_t EN:1;
        };
        uint8_t CTRL;
    } CTRL;
    uint8_t PERIODLOW;
    uint8_t WIDTHLOW;
    union {
        struct {
            uint8_t PERIODHIGH:4;
            uint8_t WIDTHHIGH:4;
        };
        uint8_t BYTE;
    } EXT;
} PWM1_SFRS_t;

/**
 * @brief The starting address of PWM1 SFRS.
 */
#define PWM1_SFRS ((__IO PWM1_SFRS_t *) (ASIC_7B_BASE + 0x48))

#define E_PWM1_PRESCALE_DIV1            0
#define E_PWM1_PRESCALE_DIV2            1
#define E_PWM1_PRESCALE_DIV4            2
#define E_PWM1_PRESCALE_DIV8            3
#define E_PWM1_PRESCALE_DIV32           4
#define E_PWM1_PRESCALE_DIV256          5
#define E_PWM1_PRESCALE_DIV8192         6
#define E_PWM1_PRESCALE_DIV262144       7

/**
 * @brief Set period for PWM1
 *
 * @param period The PWM period in clock counts.
 */
static __INLINE void f_PWM1_SetPeriod(uint32_t period)
{
    PWM1_SFRS->PERIODLOW = period & 0xFF;
    PWM1_SFRS->EXT.WIDTHHIGH = (period >> 8) & 0x0F;  
}

/**
 * @brief Set width for PWM1
 *
 * @param width The PWM width in clock counts.
 */
static __INLINE void f_PWM1_SetWidth(uint32_t width)
{
    PWM1_SFRS->WIDTHLOW = width & 0xFF;
    PWM1_SFRS->EXT.PERIODHIGH = (width >> 8) & 0x0F;  
}

/**
 * @brief Set clock pre-scaler for PWM1
 *
 * @param div The PWM clock divider setting.
 */
static __INLINE void f_PWM1_SetPreScaler(uint8_t div)
{
    PWM1_SFRS->CTRL.PRESC = div;
}

/**
 * @brief Enable on inverter for PWM1
 */
static __INLINE void f_PWM1_InvertOn(void)
{
    PWM1_SFRS->CTRL.INV = 1;
}

/**
 * @brief Disable on inverter for PWM1
 */
static __INLINE void f_PWM1_InvertOff(void)
{
    PWM1_SFRS->CTRL.INV = 0;
}

/**
 * @brief Enable PWM1
 */
static __INLINE void f_PWM1_Enable(void)
{
    PWM1_SFRS->CTRL.EN = 1;
}

/**
 * @brief Disable PWM1
 */
static __INLINE void f_PWM1_Disable(void)
{
    PWM1_SFRS->CTRL.EN = 0;
}

/**
 * @brief A structure to represent Special Function Registers for PWM2.
 */
typedef struct {
    union {
        struct {
            uint8_t PRESC:3;
            uint8_t Reserved1:1;
            uint8_t INV:1;
            uint8_t Reserved2:2;
            uint8_t EN:1;
        };
        uint8_t CTRL;
    } CTRL;
    uint8_t PERIODLOW;
    uint8_t WIDTHLOW;
    union {
        struct {
            uint8_t PERIODHIGH:4;
            uint8_t WIDTHHIGH:4;
        };
        uint8_t BYTE;
    } EXT;
} PWM2_SFRS_t;

/**
 * @brief The starting address of PWM2 SFRS.
 */
#define PWM2_SFRS ((__IO PWM2_SFRS_t *) (ASIC_7B_BASE + 0x4C))

#define E_PWM2_PRESCALE_DIV1            0
#define E_PWM2_PRESCALE_DIV2            1
#define E_PWM2_PRESCALE_DIV4            2
#define E_PWM2_PRESCALE_DIV8            3
#define E_PWM2_PRESCALE_DIV32           4
#define E_PWM2_PRESCALE_DIV256          5
#define E_PWM2_PRESCALE_DIV8192         6
#define E_PWM2_PRESCALE_DIV262144       7

/**
 * @brief Set period for PWM2
 *
 * @param period The PWM period in clock counts.
 */
static __INLINE void f_PWM2_SetPeriod(uint32_t period)
{
    PWM2_SFRS->PERIODLOW = period & 0xFF;
    PWM2_SFRS->EXT.WIDTHHIGH = (period >> 8) & 0x0F;  
}

/**
 * @brief Set width for PWM2
 *
 * @param width The PWM width in clock counts.
 */
static __INLINE void f_PWM2_SetWidth(uint32_t width)
{
    PWM2_SFRS->WIDTHLOW = width & 0xFF;
    PWM2_SFRS->EXT.PERIODHIGH = (width >> 8) & 0x0F;  
}

/**
 * @brief Set clock pre-scaler for PWM2
 *
 * @param div The PWM clock divider setting.
 */
static __INLINE void f_PWM2_SetPreScaler(uint8_t div)
{
    PWM2_SFRS->CTRL.PRESC = div;
}

/**
 * @brief Enable on inverter for PWM2
 */
static __INLINE void f_PWM2_InvertOn(void)
{
    PWM2_SFRS->CTRL.INV = 1;
}

/**
 * @brief Disable on inverter for PWM2
 */
static __INLINE void f_PWM2_InvertOff(void)
{
    PWM2_SFRS->CTRL.INV = 0;
}

/**
 * @brief Enable PWM2
 */
static __INLINE void f_PWM2_Enable(void)
{
    PWM2_SFRS->CTRL.EN = 1;
}

/**
 * @brief Disable PWM2
 */
static __INLINE void f_PWM2_Disable(void)
{
    PWM2_SFRS->CTRL.EN = 0;
}

#endif /* __PWM_SFRS_H__ */
