/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main.c
 */

#include "adc_device.h"
#include "clock_device.h"
#include "flash_device.h"
#include "gpio_device.h"
#include "i2c_device.h"
#include "uart_device.h"
#include "spi_device.h"
#include "irq_device.h"
#include "led_device.h"
#include "pmu_device.h"
#include "pwm_device.h"
#include "rtc_device.h"
#include "systick_device.h"
#include "timer_device.h"
#include "wdt_device.h"

#include "mpu9250.h"
#include "tmp007.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER0_Interval         512000                  /* 40ms */
#define TIMER1_Interval         TIMER0_Interval * 25    /* 1s */
#define TIMER2_Interval         TIMER0_Interval * 50    /* 2s */

#define BUFLEN                  16

#define MSG_START               0xBEEF
#define MSG_ID_SYNC             0x55             
#define MSG_ID_MOTION           0x01 
#define MSG_ID_THERM            0x02             

typedef struct {
    uint16_t msg_start;
    uint8_t msg_id;
    uint8_t msg_len;
} msg_head_t;

typedef struct {
    uint8_t accel_x_h;
    uint8_t accel_x_l;
    uint8_t accel_y_h;
    uint8_t accel_y_l;
    uint8_t accel_z_h;
    uint8_t accel_z_l;
    uint8_t temp_h;    
    uint8_t temp_l;    
    uint8_t gyro_x_h;
    uint8_t gyro_x_l;
    uint8_t gyro_y_h;
    uint8_t gyro_y_l;
    uint8_t gyro_z_h;
    uint8_t gyro_z_l;
} motion_data_raw_t;

typedef struct {
    int16_t accel_x;
    int16_t accel_y;
    int16_t accel_z;
    int16_t gyro_x;
    int16_t gyro_y;
    int16_t gyro_z;
} motion_data_t;

typedef struct {
    uint16_t t_die;
    uint16_t t_obj;
} therm_data_raw_t;

typedef struct {
    int16_t t_die;
    int16_t t_obj;
} therm_data_t;

typedef struct {
    msg_head_t head;
    therm_data_t data;
    uint16_t crc;
} therm_msg_t;

typedef struct {
    msg_head_t head;
    motion_data_t data;
    uint16_t crc;
} motion_msg_t;

typedef struct {
    msg_head_t head;
} sync_msg_t;

/************** Local Variables ***********************************************/
static SysTick_Device_t *systick = NULL;
static I2C_Device_t *i2c_master = NULL;
static UART_Device_t *uart = NULL;
static SPI_Device_t *spi = NULL;

/************** Global Variables **********************************************/
/************** Functions *****************************************************/
static int32_t i2c_check_timeout(I2C_Device_t *dev, uint32_t timeout)
{
    uint64_t time;
    time = SysTick_devGetElapsedMs(systick);
    while (I2C_devIsInProgress(dev)) {
        if ((SysTick_devGetElapsedMs(systick) - time) > timeout) {
            I2C_devReset(dev);
            return -ETIMEOUT;
        }
    }
    return 0;
}

static int32_t uart_check_timeout(UART_Device_t *dev, uint32_t timeout)
{
    uint64_t time;
    time = SysTick_devGetElapsedMs(systick);
    while (UART_devDataPendingTX(dev)) {
        if ((SysTick_devGetElapsedMs(systick) - time) > timeout) {
            UART_devResetDevice(dev);
            return -ETIMEOUT;
        }
    }
    return 0;
}

/**
 * @brief I2C master initialization.
 *
 * This is a demo code to initialize the I2C as master.
 *
 * @param baud The enumeration of baud rate.
 * @return 0 for success or error code upon on a failure.
 */
static int32_t i2c_master_init(uint8_t baud)
{
    int32_t ret;
    GPIO_Device_t *gpiod = NULL;

    gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return -ENODEV;
    
    i2c_master = I2C_devGetDevice("I2C");
    if(!i2c_master)
        return -ENODEV;
    I2C_Device_MasterInfo_t i2c_info = {
        .SlaveAddrLen = 7,
        .FilterEnable = false,
        .Speed = baud,
    };
    
    GPIO_devInit(gpiod, 0, GPIO_MODE_INPUT | GPIO_MODE_OUTPUT);
    GPIO_devInit(gpiod, 1, GPIO_MODE_INPUT | GPIO_MODE_OUTPUT);
    GPIO_ConfigI2C(E_GPIO_PIN_MODE_ON, E_GPIO_I2C_PULLIP_10K);
   
    ret = I2C_devMasterInit(i2c_master, &i2c_info);
    return ret;
}

/**
 * @brief I2C master register read.
 *
 * This is a demo code to use the I2C master register read APIs.
 *
 * @param slave The slave address of I2C device.
 * @param addr The register address in the I2C slave device.
 * @param len The number of bytes to be read from or write to I2C slave device.
 * @param buf The pointer to the data buffer for received data.
 * @param timeout The time out threshold in ms.
 * @return 0 for success or error code upon on a failure.
 */
static int32_t i2c_master_register_read(uint8_t slave, uint8_t addr,
    uint8_t *buf, uint8_t len, uint32_t timeout)
{
    int32_t ret;

    ret = I2C_devMasterRegisterReadRequest(i2c_master, slave, &addr, 1, len);
    if (ret)
        return ret;
    ret = i2c_check_timeout(i2c_master, timeout);
    if (ret)
        return ret;
    ret = I2C_devMasterReadResult(i2c_master, buf, len);
    if (ret)
        return ret;
    ret = i2c_check_timeout(i2c_master, timeout);
    if (ret)
        return ret;
    return 0;
}

static void convert_motion_data(motion_data_raw_t *raw, motion_msg_t *msg)
{
    msg->head.msg_start = MSG_START;
    msg->head.msg_id = MSG_ID_MOTION;
    msg->head.msg_len = sizeof(motion_data_t);
       
    msg->data.accel_x = (raw->accel_x_h << 8) | raw->accel_x_l;
    msg->data.accel_y = (raw->accel_y_h << 8) | raw->accel_y_l;
    msg->data.accel_z = (raw->accel_z_h << 8) | raw->accel_z_l;
    msg->data.gyro_x = (raw->gyro_x_h << 8) | raw->gyro_x_l;
    msg->data.gyro_y = (raw->gyro_y_h << 8) | raw->gyro_y_l;
    msg->data.gyro_z = (raw->gyro_z_h << 8) | raw->gyro_z_l;
    msg->crc = 0;
}

static uint16_t swape_bytes(uint16_t x)
{
    uint16_t hibyte, lobyte;
    hibyte = (x & 0xff00) >> 8;
    lobyte = (x & 0xff);
    return (lobyte << 8 | hibyte);
}

static int16_t convert_to_celsius(int16_t x)
{
    int16_t data;

    if (x > 0)
        data = x >> 2;
    else
        data = (x >> 2) | 0xC000;
    return (data / 32);
}

static void convert_therm_data(therm_data_raw_t *raw, therm_msg_t *msg)
{
    therm_data_t data;

    msg->head.msg_start = MSG_START;
    msg->head.msg_id = MSG_ID_THERM;
    msg->head.msg_len = sizeof(therm_data_t);

    data.t_die = swape_bytes(raw->t_die);
    data.t_obj = swape_bytes(raw->t_obj);
    msg->data.t_die = convert_to_celsius(data.t_die);
    msg->data.t_obj = convert_to_celsius(data.t_obj);
    msg->crc = 0;
}

/**
 * @brief UART initialization.
 *
 * This is a demo code to initialize the UART device driver.
 *
 * @param baud The enumeration of baud rate.
 * @return 0 for success or error code upon on a failure.
 */
static int32_t uart_init(UART_devBaudRate_t baud)
{
    int32_t ret;

    /* Config GPIO to UART mode */
    GPIO_ConfigUART(E_GPIO_PIN_MODE_ON, E_GPIO_PIN_SWAP, E_GPIO_PIN_NOINVERT);
    /* Request UART device */
    uart = UART_devRequest("UART");
    UART_Dev_LineInitData_t uart_config = {
        .DataSize = 8,
        .StopBits = 1,
        .ParityBit = UART_DEV_PARITY_BIT_ODD,
        .ParityEn = 0,
        .LoopBackEn = 0,
        .BreakEn = 0,
        .StickyParityEn = 0,
    };
    /* Initialize UART device */
    ret = UART_devInit(uart, baud, &uart_config);
    return ret;
}

static int32_t uart_send_data(uint8_t *data, uint8_t size)
{
    int32_t ret;

    ret = UART_devSendByte(uart, data, size);
    if (!ret)
        return uart_check_timeout(uart, 100);
    return ret;
}

static int32_t spi_init(uint8_t clk)
{
    int32_t ret;

    /* Initialize GPIO pins for SPI mode */
    GPIO_Device_t *gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return -ENODEV;
    GPIO_ConfigSPI(E_GPIO_PIN_MODE_ON);
    GPIO_devInit(gpiod, 2, GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 3, GPIO_MODE_OUTPUT);
    GPIO_devInit(gpiod, 4, GPIO_MODE_OUTPUT);
    GPIO_devInit(gpiod, 5, GPIO_MODE_OUTPUT);

    /* Initialize SPI driver */
    spi = SPI_devRequest("SPI");
    if(!spi)
        return -ENODEV;
    SPI_Device_init_t spi_init = {      
        .Mode = SPI_DEV_MODE_MASTER,
        .ClockDiv = clk,
        .Phase = SPI_DEV_CLOCK_PHASE0,
        .Polarity = SPI_DEV_CLOCK_POLARITY0,
        .IntDlyCnt = 0,
    };
    ret = SPI_devInit(spi, &spi_init);
    return ret;
}

/************** MAIN ****************************************************/
void main_demo (void)
{
    int32_t ret;
    uint64_t time, time0, time1;

    motion_data_raw_t motion_data_raw;
    therm_data_raw_t therm_data_raw;
    sync_msg_t sync_msg;
    motion_msg_t motion_msg;
    therm_msg_t therm_msg;

    sync_msg.head.msg_start = MSG_START;
    sync_msg.head.msg_id = MSG_ID_SYNC;
    sync_msg.head.msg_len = 0;
    
    /* Initialize system clock */
    Clock_devRCCalibration();
    Clock_devRCClockInit();
    Clock_devEnableCrystalClock();

    /* Initialize power mode */
    PMU_devSetPowerModeLDO();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    ret = SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);
    if(ret)
        return;
 
    /* Initialize I2C driver */
    ret = i2c_master_init(I2C_DEVICE_SPEED_100K);
    if(ret)
        return;
    /* Initialize UART driver */
    ret = uart_init(UART_DEV_BAUDRATE_57600);
    if(ret)
        return;
    ret = spi_init(SPI_DEV_CLOCK_DIV8);
    if(ret)
        return;

    time0 = SysTick_devGetElapsedMs(systick);
    while (1) {      
        SPI_devReadData(spi, MPU9250_SPI_READ_CMD | ACCEL_XOUT_H, (uint8_t *)&motion_data_raw, 14);
        convert_motion_data(&motion_data_raw, &motion_msg);
        uart_send_data((uint8_t *)&motion_msg, sizeof(motion_msg));

        time = SysTick_devGetElapsedMs(systick);
        if ((time - time0) > 500) {
            time0 = time;
            i2c_master_register_read(TMP007_ADDRESS, TMP007_LOCAL_TEMPERATURE, (uint8_t *)&therm_data_raw.t_die, 2, 100);
            i2c_master_register_read(TMP007_ADDRESS, TMP007_OBJECT_TEMPERATURE, (uint8_t *)&therm_data_raw.t_obj, 2, 100);
            convert_therm_data(&therm_data_raw, &therm_msg);
            uart_send_data((uint8_t *)&therm_msg, sizeof(therm_msg));
        }

        if ((time - time1) > 1000) {
            time1 = time;
            uart_send_data((uint8_t *)&sync_msg, sizeof(sync_msg));
        }
    } /* while (1) */
} /* main() */

/***************************** End of File ************************************/