/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_i2c.c
 */

#include "clock_device.h"
#include "gpio_device.h"
#include "i2c_device.h"
#include "systick_device.h"
#include "pmu_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)
#define I2C_SLAVE_ADDR 0x7A
#define REG_ADDR_LEN 2
#define BUFLEN 16

/************** Local Variables ***********************************************/
static SysTick_Device_t *systick = NULL;
static I2C_Device_t *i2c_master = NULL;
        
static uint8_t read_buf[BUFLEN] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
static uint8_t write_buf[BUFLEN] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
static uint16_t reg_address = 0x2020;


/************** Global Variables **********************************************/
/************** Functions *****************************************************/
static int32_t check_timeout(I2C_Device_t *dev, uint32_t timeout)
{
    uint64_t time;
    time = SysTick_devGetElapsedMs(systick);
    while (I2C_devIsInProgress(dev)) {
        if ((SysTick_devGetElapsedMs(systick) - time) > timeout) {
            I2C_devReset(dev);
            return -ETIMEOUT;
        }
    }
    return 0;
}

/**
 * @brief I2C master initialization.
 *
 * This is a demo code to initialize the I2C as master.
 *
 * @param baud The enumeration of baud rate.
 * @return 0 for success or error code upon on a failure.
 */
static int32_t i2c_master_init(uint8_t baud)
{
    int32_t ret;
    GPIO_Device_t *gpiod = NULL;

    gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return -ENODEV;
    
    i2c_master = I2C_devGetDevice("I2C");
    if(!i2c_master)
        return -ENODEV;
    I2C_Device_MasterInfo_t i2c_info = {
        .SlaveAddrLen = 7,
        .FilterEnable = false,
        .Speed = baud,
    };
    
    GPIO_devInit(gpiod, 0, GPIO_MODE_INPUT | GPIO_MODE_OUTPUT);
    GPIO_devInit(gpiod, 1, GPIO_MODE_INPUT | GPIO_MODE_OUTPUT);
    GPIO_ConfigI2C(E_GPIO_PIN_MODE_ON, E_GPIO_I2C_PULLIP_10K);
   
    ret = I2C_devMasterInit(i2c_master, &i2c_info);
    return ret;
}

/**
 * @brief I2C master register read.
 *
 * This is a demo code to use the I2C master register read APIs.
 *
 * @param byte_count The number of bytes to be read from or write to I2C slave device.
 * @param timeout The time out threshold in ms.
 * @return 0 for success or error code upon on a failure.
 */
static int32_t i2c_master_register_read(uint8_t byte_count, uint32_t timeout)
{
    int32_t ret;

    ret = I2C_devMasterRegisterReadRequest(i2c_master, I2C_SLAVE_ADDR, (uint8_t *)&reg_address, REG_ADDR_LEN, byte_count);
    if (ret)
        return ret;
    ret = check_timeout(i2c_master, timeout);
    if (ret)
        return ret;
    ret = I2C_devMasterReadResult(i2c_master, &read_buf[0], byte_count);
    if (ret)
        return ret;
    ret = check_timeout(i2c_master, timeout);
    if (ret)
        return ret;
    return 0;
}


/**
 * @brief I2C master read and write test.
 *
 * This is a demo code to use the I2C master read and write APIs.
 *
 * @param count The number of bytes to be read from or write to I2C slave device.
 * @param timeout The time out threshold in ms.
 * @return 0 for success or error code upon on a failure.
 */
static int32_t i2c_master_read_and_write(uint8_t count, uint32_t timeout)
{
    int32_t ret;
    
    ret = I2C_devMasterSendData(i2c_master, I2C_SLAVE_ADDR, &write_buf[0], count);
    if (ret)
        return ret;
    ret = check_timeout(i2c_master, timeout);
    if (ret)
        return ret;
    ret = I2C_devMasterReadRequest(i2c_master, I2C_SLAVE_ADDR, count);
    if (ret)
        return ret;
    ret = check_timeout(i2c_master, timeout);
    if (ret)
        return ret;
    ret = I2C_devMasterReadResult(i2c_master, &read_buf[0], count);

    return ret;
}

/************** MAIN ****************************************************/
void main_i2c (void)
{
    int32_t ret;

    /* Initialize system clock */
    Clock_devRCCalibration();
    Clock_devRCClockInit();
    Clock_devEnableCrystalClock();

    /* Initialize power mode */
    PMU_devSetPowerModeLDO();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, 0);

    /* Initialize i2c driver */
    ret = i2c_master_init(I2C_DEVICE_SPEED_400K);
    if (ret)
        return;

    while (1) {
        /* Read data from I2C slave in register mode */
        ret = i2c_master_register_read(16, 100);
        /* Read and write data from I2C slave */
        ret = i2c_master_read_and_write(16, 100);
        SysTick_devDelayMs(systick, 500);
    } /* while (1) */
} /* main() */

/***************************** End of File ************************************/