/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_i2c_slave.c
 */

#include "clock_device.h"
#include "gpio_device.h"
#include "i2c_device.h"
#include "systick_device.h"
#include "pmu_device.h"
#include "errno.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)
#define I2C_SLAVE_ADDR 0x7A
#define REG_ADDR_LEN 2
#define BUFLEN 16

/************** Local Variables ***********************************************/
static SysTick_Device_t *systick = NULL;
static I2C_Device_t *i2c_slave = NULL;
        
static uint8_t read_buf[BUFLEN] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
static uint8_t write_buf[BUFLEN] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};

/************** Global Variables **********************************************/
/************** Functions *****************************************************/
/**
 * @brief I2C slave initialization.
 *
 * This is a demo code to initialize the I2C as master.
 *
 * @param baud The enumeration of baud rate.
 * @return 0 for success or error code upon on a failure.
 */
static int32_t i2c_slave_init(uint8_t baud)
{
    int32_t ret;
    GPIO_Device_t *gpiod = NULL;

    /* Initialize GPIO pins for I2C */
    gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return -ENODEV;
    GPIO_devInit(gpiod, 0, GPIO_MODE_INPUT | GPIO_MODE_OUTPUT);
    GPIO_devInit(gpiod, 1, GPIO_MODE_INPUT | GPIO_MODE_OUTPUT);
    GPIO_ConfigI2C(E_GPIO_PIN_MODE_ON, E_GPIO_I2C_PULLIP_10K);
    
    /* Initialize I2C driver as a slave */
    i2c_slave = I2C_devGetDevice("I2C");
    if(!i2c_slave)
        return -ENODEV;
    I2C_Device_SlaveInfo_t i2c_info = {
        .SlaveAddrLen = 7,
        .FilterEnable = false,
        .Speed = baud,
        .GeneralCallEnable = false,
        .SlaveAddr = 0x7A,   
        .SlaveAddrMask = 0x03,
    };
    ret = I2C_devSlaveInit(i2c_slave, &i2c_info);
    return ret;
}
/**
 * @brief I2C Slave read and write.
 *
 * This is a demo code to use the I2C slave APIs.
 *
 * @param count The number of bytes to be read from or write to I2C slave device.
 * @return 0 for success or error code upon on a failure.
 */
static int32_t i2c_slave_read_and_write(uint8_t count)
{
    int32_t ret;

    /* Wait for data from I2C master */
    while (I2C_devSlaveDataReceived(i2c_slave) < count);
    /* Read data from I2C device driver */
    ret = I2C_devSlaveReadResult(i2c_slave, &write_buf[0], count);
    /* Restart device driver for next cycle */
    I2C_devSlaveRestart(i2c_slave);

    /* Set data bufer to be sent to I2C master */
    ret = I2C_devSlaveSetResponseData(i2c_slave, &read_buf[0], count);
    /* Wait data to be sent */
    while (I2C_devSlaveDataResponsed(i2c_slave) < count);
    /* Restart device driver for next cycle */
    I2C_devSlaveRestart(i2c_slave);
    return ret;
}

/************** MAIN ****************************************************/
void main_i2c_slave (void)
{
    int32_t ret;

    /* Initialize system clock */
    Clock_devRCCalibration();
    Clock_devRCClockInit();
    Clock_devEnableCrystalClock();

    /* Initialize power mode */
    PMU_devSetPowerModeLDO();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, 0);

    /* Initialize i2c driver */
    ret = i2c_slave_init(I2C_DEVICE_SPEED_400K);
    if (ret)
        return;

    while (1) {
        /* Read and write data from I2C master */
        ret = i2c_slave_read_and_write(16);
        SysTick_devDelayMs(systick, 500);
    } /* while (1) */
} /* main() */

/***************************** End of File ************************************/